/*****************************************************************
[HEADER] 

This program generates Volumes and corresponding four dimensional
sinograms of line integrals.

Version 1.0


Currently two basic functions can be used. Either a uniform ball or a
Gaussian 3D bell. Both can be scaled, rotated, and shifted and put
together with an arbitrary number of objects.

Peter Toft and Peter Philipsen

Electronics Institute, Building 349,

Technical University of Denmark,

2800 Lyngby

Denmark

ptoft,pap@ei.dtu.dk
*****************************************************************/
#include "Recon3D.h"

#define epsangle 1e-5

struct info3D
{
  float off_x,off_y,off_z;
  float rot_x,rot_y,rot_z;
  float scal_x,scal_y,scal_z;
  float pow;
  int phtype;
};

int GenerateVolume,GenerateSinogram,NumberOfShapes;
MultiStruct *Volume,*Sinogram;
int XSamples,YSamples,ZSamples,USamples,VSamples,ThetaSamples,PhiSamples;
float Xmin,Ymin,Zmin,DeltaX,DeltaY,DeltaZ,Umin,Vmin,PhiLimit;
float DeltaPhi,DeltaU,DeltaV;
struct info3D *e;
char OutFileName[200];
char *IniBuffer;
char typear[3][50]={"Not        ","Ellipse    ","Gaus. Bell "};

/**************************************************************************
[NAME]
init

[SYNOPSIS]
void init(void)

[DESCRIPTION]
The function will initialize and read the {\tt .ini}-file.

[REVISION]
6/6-95 Peter Toft 
**************************************************************************/
void init(void)
{
  int i;
  char Temp[100],Tempi[100];
  float pidiv180=M_PI/180;

  if (!(GetArg(IniBuffer,"DebugLevel",Temp))) DebugLevel=_DNormal;
  else if (strstr(Temp,"Normal"))             DebugLevel=_DNormal;
  else if (strstr(Temp,"Debug"))              DebugLevel=_DDebug;
  else if (strstr(Temp,"Silent"))             DebugLevel=_DSilent;
  else if (strstr(Temp,"NoLog"))              DebugLevel=_DNoLog;
  else if (strstr(Temp,"HardCore"))           DebugLevel=_DHardCore;
  else Error("Unknown DebugLevel: '%s'",Temp);   

  strcpy(LogFileName,"3D_RadonAna");
  OpenLog(LogFileName);  

  Print(_DNormal,"\n**************************************\n");
  Print(_DNormal,"* 3D_RadonAna, rev 1.0               *\n");
  Print(_DNormal,"* Peter Toft 1996                    *\n");
  Print(_DNormal,"**************************************\n\n");

  if (!(GetArg(IniBuffer,"OutFileName",OutFileName)))
    Error("`OutFileName' entry missing in INI file");
  Print(_DNormal,"`OutFileName' entry is set to %s\n",OutFileName);

  if (!(GetArg(IniBuffer,"XSamples",Temp))) 
    Error("`XSamples' entry missing in INI file");
  sscanf(Temp,"%i",&XSamples);
  Print(_DNormal,"`XSamples' entry is set to %i\n",XSamples);
  
  if (!(GetArg(IniBuffer,"YSamples",Temp)))
  {
    YSamples=XSamples;
    Print(_DNormal,"`YSamples' entry missing in INI file. Setting it to %i\n",
          YSamples);
  }
  else
  {
    sscanf(Temp,"%i",&YSamples);
    Print(_DNormal,"`YSamples' entry is set to %i\n",YSamples);
  }
  
  if (!(GetArg(IniBuffer,"ZSamples",Temp)))
  {
    ZSamples=XSamples;
    Print(_DNormal,"`ZSamples' entry missing in INI file. Setting it to %i\n",
          ZSamples);
  }
  else
  {
    sscanf(Temp,"%i",&ZSamples);
    Print(_DNormal,"`ZSamples' entry is set to %i\n",ZSamples);
  }
  

  if (!(GetArg(IniBuffer,"DeltaX",Temp)))
    Error("`DeltaX' entry missing in INI file");
  sscanf(Temp,"%f",&DeltaX);
  Print(_DNormal,"`DeltaX' entry is set to %f\n",DeltaX);

  if (!(GetArg(IniBuffer,"DeltaY",Temp)))
  {
    DeltaY=DeltaX;
    Print(_DNormal,"`DeltaY' entry missing in INI file. Setting it to %f\n",
          DeltaY);
  }
  else
  {
    sscanf(Temp,"%f",&DeltaY);
    Print(_DNormal,"`DeltaY' entry is set to %f\n",DeltaY);
  }

  if (!(GetArg(IniBuffer,"DeltaZ",Temp)))
  {
    DeltaZ=DeltaX;
    Print(_DNormal,"`DeltaZ' entry missing in INI file. Setting it to %f\n",
          DeltaY);
  }
  else
  {
    sscanf(Temp,"%f",&DeltaZ);
    Print(_DNormal,"`DeltaZ' entry is set to %f\n",DeltaZ);
  }
 
  if (!(GetArg(IniBuffer,"Xmin",Temp)))
  {
    Xmin=(1-XSamples)*0.5*DeltaX;
    Print(_DNormal,"`Xmin' entry missing in INI file. Setting it to %f\n",
          Xmin);
  }
  else
  {
    sscanf(Temp,"%f",&Xmin);
    Print(_DNormal,"`Xmin' entry is set to %f\n",Xmin);
  }
  
  if (!(GetArg(IniBuffer,"Ymin",Temp)))
  {
    Ymin=(1-YSamples)*0.5*DeltaX;
    Print(_DNormal,"`Ymin' entry missing in INI file. Setting it to %f\n",Ymin);
  }
  else
  {
    sscanf(Temp,"%f",&Ymin);
    Print(_DNormal,"`Ymin' entry is set to %f\n",Ymin);
  }
  
  if (!(GetArg(IniBuffer,"Zmin",Temp)))
  {
    Zmin=(1-ZSamples)*0.5*DeltaZ;
    Print(_DNormal,"`Zmin' entry missing in INI file. Setting it to %f\n",
          Zmin);
  }
  else
  {
    sscanf(Temp,"%f",&Zmin);
    Print(_DNormal,"`Zmin' entry is set to %f\n",Zmin);
  }

  if (!(GetArg(IniBuffer,"USamples",Temp)))
    Error("`USamples' entry missing in INI file");
  sscanf(Temp,"%i",&USamples);
  Print(_DNormal,"`USamples' entry is set to %i\n",USamples);

  if (!(GetArg(IniBuffer,"VSamples",Temp)))
  {
    VSamples=USamples;
    Print(_DNormal,"`VSamples' entry missing in INI file. Setting it to %i\n",
          VSamples);
  }
  else
  {
    sscanf(Temp,"%i",&VSamples);
    Print(_DNormal,"`VSamples' entry is set to %i\n",VSamples);
  }
  
  if (!(GetArg(IniBuffer,"ThetaSamples",Temp)))
    Error("`ThetaSamples' entry missing in INI file");
  sscanf(Temp,"%i",&ThetaSamples);
  Print(_DNormal,"`ThetaSamples' entry is set to %i\n",ThetaSamples);

  if (!(GetArg(IniBuffer,"PhiSamples",Temp)))
    Error("`PhiSamples' entry missing in INI file");
  sscanf(Temp,"%i",&PhiSamples);
  Print(_DNormal,"`PhiSamples' entry is set to %i\n",PhiSamples);

  if (!(GetArg(IniBuffer,"PhiLimit",Temp)))
    Error("`PhiLimit' entry missing in INI file");
  sscanf(Temp,"%f",&PhiLimit);
  Print(_DNormal,"`PhiLimit' entry is set to %i degrees\n",PhiLimit);
  PhiLimit*=pidiv180;

  DeltaPhi=2*PhiLimit/(PhiSamples-1);
  Print(_DNormal,"`DeltaPhi' entry is set to %f\n",DeltaPhi);

  if (!(GetArg(IniBuffer,"DeltaU",Temp)))
    Error("`DeltaU' entry missing in INI file");
  sscanf(Temp,"%f",&DeltaU);
  Print(_DNormal,"`DeltaU' entry is set to %f\n",DeltaU);

  if (!(GetArg(IniBuffer,"DeltaV",Temp)))
  {
    DeltaV=DeltaU;
    Print(_DNormal,"`DeltaV' entry missing in INI file. Setting it to %f\n",
          DeltaV);
  }
  else
  {
    sscanf(Temp,"%f",&DeltaV);
    Print(_DNormal,"`DeltaV' entry is set to %i\n",DeltaV);
  }

  
  if (!(GetArg(IniBuffer,"Umin",Temp)))
  {
    Umin=(1-USamples)*0.5*DeltaU;
    Print(_DNormal,"`Umin' entry missing in INI file. Setting it to %f\n",
          Umin);
  }
  else
  {
    sscanf(Temp,"%f",&Umin);
    Print(_DNormal,"`Umin' entry is set to %f\n",Umin);
  }


  
  if (!(GetArg(IniBuffer,"Vmin",Temp)))
  {
    Vmin=(1-VSamples)*0.5*DeltaV;
    Print(_DNormal,"`Vmin' entry missing in INI file. Setting it to %f\n",
          Vmin);
  }
  else
  {
    sscanf(Temp,"%f",&Vmin);
    Print(_DNormal,"`Vmin' entry is set to %f\n",Vmin);
  }

  if (!(GetArg(IniBuffer,"NumberOfShapes",Temp)))
    Error("`NumberOfShapes' entry missing in INI file");
  sscanf(Temp,"%i",&NumberOfShapes);
  Print(_DNormal,"`NumberOfShapes' entry is set to %i\n",NumberOfShapes);

  e=(struct info3D *)malloc(NumberOfShapes*sizeof(struct info3D));

  for (i=0; i<NumberOfShapes; i++)
  {
    sprintf(Tempi,"Shape%i",i);
    if (!(GetArg(IniBuffer,Tempi,Temp)))
    {
      Print(_DNormal,
            "Shape Number %i is missing in the INI file. Specify '%s'\n",
            i,Tempi);
      Error("Terminating Program\n");
    }
    sscanf(Temp,"%i %f %f %f %f %f %f %f %f %f %f", 
	   &e[i].phtype, 
           &e[i].off_x, &e[i].off_y, &e[i].off_z,
           &e[i].rot_x, &e[i].rot_y, &e[i].rot_z,
           &e[i].scal_x, &e[i].scal_y, &e[i].scal_z,
	   &e[i].pow);

    Print(_DNormal,"\n%2i :    type=%s  pow=%7.3f\n",
          i,typear[e[i].phtype],e[i].pow);
    Print(_DNormal,"      off_x0=%7.3f    off_y0=%7.3f    off_z0=%7.3f\n",
         e[i]. off_x,e[i].off_y,e[i].off_z);
    Print(_DNormal,"      rot_x0=%7.3f    rot_y0=%7.3f    rot_z0=%7.3f\n",
          e[i].rot_x,e[i].rot_y,e[i].rot_z);
    Print(_DNormal,"     scal_x0=%7.3f   scal_y0=%7.3f   scal_z0=%7.3f\n",
          e[i].scal_x,e[i].scal_y,e[i].scal_z);

    e[i].rot_x*=pidiv180;
    e[i].rot_y*=pidiv180;
    e[i].rot_z*=pidiv180;
  }

  if (!(GetArg(IniBuffer,"GenerateVolume",Temp)))
  {
    GenerateVolume=1;
    Print(_DNormal,
          "`GenerateVolume' entry missing in INI file. Setting it to %i\n",
          GenerateVolume);
  }
  else
  {
    sscanf(Temp,"%i",&GenerateVolume);
    Print(_DNormal,"`GenerateVolume entry is set to %i\n",GenerateVolume);
  }
  
  if (!(GetArg(IniBuffer,"GenerateSinogram",Temp)))
  {
    GenerateSinogram=1;
    Print(_DNormal,
          "`GenerateSinogram' entry missing in INI file. Setting it to %i\n",
          GenerateSinogram);
  }
  else
  {
    sscanf(Temp,"%i",&GenerateSinogram);
    Print(_DNormal,"`GenerateSinogram' entry is set to %i\n",GenerateSinogram);
  }
}

void GetSphericalAngles(float *theta,
			float *phi,
			float vx,
			float vy,
			float vz)
{
  float length,sintheta,costheta;

  length=sqrt(vx*vx+vy*vy+vz*vz);
  if (fabs(length)<epsangle)
    Error("Input Vector has zero length (GetSphericalAngles)\n");

  vx/=length;
  vy/=length;
  vz/=length;

  if (vz>1-epsangle)
  {
    *theta=0;
    *phi=0;
  }
  else
    if (vz<-1+epsangle)
    {
      *phi=M_PI;
      *theta=0;
    }
    else
    {
      *theta=atan2(vy,vx);
      costheta=cos(*theta);
      sintheta=sin(*theta);
      if (fabs(sintheta)>fabs(costheta))
	*phi=atan2(vz,vy/sintheta);
      else
	*phi=atan2(vz,vx/costheta);
    }
}


/**************************************************************************
[NAME]
MakeVolume

[SYNOPSIS]
void MakeVolume(void)

[DESCRIPTION]
The function will generate the volume.

[REVISION]
6/6-95 Peter Toft 
**************************************************************************/
MultiStruct *MakeVolume(void)
{
  int *Samples;
  int shape,i,j,k;
  int shapetype;
  float a,b,c;
  float off_x0,off_y0,off_z0;
  float scal_x,scal_y,scal_z,c1,s1,c2,s2,c3,s3;
  float rotx,roty,rotz,newx,newy,newz;
  float aa,bb,cc;
  float a11,a12,a13,a21,a22,a23,a31,a32,a33,mypow;
  float *xar,*yar,*zar;
  char VolumeName[200];
  MultiStruct *My3D;

  Print(_DNormal,"MakeVolume: Starting\n");

  strcpy(VolumeName,OutFileName);
  strcat(VolumeName,"_Vol");
  
  Samples=IntVector(3);
  Samples[0]=XSamples;
  Samples[1]=YSamples;
  Samples[2]=XSamples;
  My3D=NewMultiStruct(VolumeName,3,Samples,_RealArray);
  My3D->Delta[0]=DeltaX;
  My3D->Delta[1]=DeltaY;
  My3D->Delta[2]=DeltaZ;
  My3D->Min[0]=Xmin;
  My3D->Min[1]=Ymin;
  My3D->Min[2]=Zmin;
  xar=FloatVector(XSamples);
  yar=FloatVector(YSamples);
  zar=FloatVector(ZSamples);
  
  for (i=0;i<Samples[0];i++)
    xar[i]=My3D->Min[0]+i*My3D->Delta[0];
  for (i=0;i<Samples[1];i++)
    yar[i]=My3D->Min[1]+i*My3D->Delta[1];
  for (i=0;i<Samples[2];i++) 
    zar[i]=My3D->Min[2]+i*My3D->Delta[2];
    
  for (shape=0;shape<NumberOfShapes;shape++)
  {
    Print(_DNormal,"MakeVolume: %3i (%3i)\r",shape,NumberOfShapes);
    off_x0=e[shape].off_x;
    off_y0=e[shape].off_y;
    off_z0=e[shape].off_z;
    rotx=e[shape].rot_x;
    roty=e[shape].rot_y;
    rotz=e[shape].rot_z;
    scal_x=e[shape].scal_x;
    scal_y=e[shape].scal_y;
    scal_z=e[shape].scal_z;
    mypow=e[shape].pow;
    shapetype=e[shape].phtype;
    c1=cos(rotz);
    s1=sin(rotz);
    c2=cos(roty);
    s2=sin(roty);
    c3=cos(rotx);
    s3=sin(rotx);
  
    a11=c1*c2/scal_x;
    a12=c2*s1/scal_x;
    a13=s2/scal_x;
    a21=-(c3*s1+c1*s2*s3)/scal_y;
    a22=(c1*c3-s1*s2*s3)/scal_y;
    a23=c2*s3/scal_y;
    a31=(s1*s3-c1*c3*s2)/scal_z;
    a32=-(c3*s1*s2+c1*s3)/scal_z;
    a33=c2*c3/scal_z;

    for (i=0;i<XSamples;i++)
    {
      a=xar[i]-off_x0;
      for (j=0;j<YSamples;j++)
      {
        b=yar[j]-off_y0;
        aa=a11*a+a12*b;
        bb=a21*a+a22*b;
        cc=a31*a+a32*b;
        for (k=0;k<ZSamples;k++)
        {
          c=zar[k]-off_z0;
          newx=aa+a13*c;
          newy=bb+a23*c;
          newz=cc+a33*c;
          switch (shapetype)
          {
            case 1 :
              if (newx*newx+newy*newy+newz*newz<1.0)
                My3D->Signal[i+XSamples*(j+YSamples*k)]+=mypow;
              break;
            case 2 :
              My3D->Signal[i+XSamples*(j+YSamples*k)]+=
                exp(-newx*newx-newy*newy-newz*newz)*mypow;
              break;
            default :
                Error("Function of wrong type detected\n");
          }            
        }
      }
    }
  }
  Print(_DNormal,"MakeVolume: Done              \n");

  Free(Samples);
  Free(xar);
  Free(yar);
  Free(zar);
  return My3D;
}


/**************************************************************************
[NAME]
MakeSinogram

[SYNOPSIS]
void MakeSinogram(void)

[DESCRIPTION]
The function will generate the 4D-Sinogram.

[REVISION]
6/6-95 Peter Toft 
**************************************************************************/
MultiStruct *MakeSinogram(void)
{
  int *Samples;
  int shapetype,shape,thetalint,philint,ulint,vlint,Off,Mult;
  int trigoff;
  float a,u,v,ur,vr,sqrtpi=sqrt(M_PI);
  float off_x0,off_y0,off_z0;
  float ntaux,ntauy,ntauz;
  float scal_x,scal_y,scal_z,c1,s1,c2,s2,c3,s3;
  float tau_x,tau_y,tau_z;
  float alpha_x,alpha_y,alpha_z;
  float beta_x,beta_y,beta_z;
  float rotx,roty,rotz;
  float a11,a12,a13,a21,a22,a23,a31,a32,a33;
  float ntheta,nphi,ncostheta,nsintheta,ncosphi,nsinphi;
  float nbeta_x,nbeta_y,nbeta_z,nalpha_x,nalpha_y,nalpha_z,mypow;
  float aubv_x,aubv_y,aubv_z,Aaubv_x,Aaubv_y,Aaubv_z,newu,newv,gamma_value;
  MultiStruct *My4D;
  TrigDef *MyTrig;
  char SinogramName[200];
  float *ar_alpha_x,*ar_alpha_y,*ar_alpha_z;
  float *ar_beta_x,*ar_beta_y,*ar_beta_z;
  float *ar_tau_x,*ar_tau_y,*ar_tau_z;

  Print(_DNormal,"MakeSinogram: Starting\n");

  strcpy(SinogramName,OutFileName);
  strcat(SinogramName,"_Sino");

  Samples=IntVector(4);
  Samples[0]=ThetaSamples;
  Samples[1]=PhiSamples;
  Samples[2]=USamples;
  Samples[3]=VSamples;

  My4D=NewMultiStruct(SinogramName,4,Samples,_RealArray);
  My4D->Delta[0]=M_PI/ThetaSamples;
  My4D->Delta[1]=DeltaPhi;
  My4D->Delta[2]=DeltaU;
  My4D->Delta[3]=DeltaV;

  My4D->Min[0]=0;
  My4D->Min[1]=-PhiLimit;
  My4D->Min[2]=Umin;
  My4D->Min[3]=Vmin;
  MyTrig=GenerateTrigDef_fromMultiStruct(My4D);

  ar_tau_x=MyTrig->tau_x;
  ar_tau_y=MyTrig->tau_y;
  ar_tau_z=MyTrig->tau_z;
  ar_alpha_x=MyTrig->alpha_x;
  ar_alpha_y=MyTrig->alpha_y;
  ar_alpha_z=MyTrig->alpha_z;
  ar_beta_x=MyTrig->beta_x;
  ar_beta_y=MyTrig->beta_y;
  ar_beta_z=MyTrig->beta_z;

  Mult=Samples[0]*Samples[1]*Samples[2];

  for (shape=0;shape<NumberOfShapes;shape++)
  {
    Print(_DNormal,"MakeSinogram: %3i (%3i)\r",shape,NumberOfShapes);
    shapetype=e[shape].phtype;
    off_x0=e[shape].off_x;
    off_y0=e[shape].off_y;
    off_z0=e[shape].off_z;
    rotx=e[shape].rot_x;
    roty=e[shape].rot_y;
    rotz=e[shape].rot_z;
    scal_x=e[shape].scal_x;
    scal_y=e[shape].scal_y;
    scal_z=e[shape].scal_z;
    mypow=e[shape].pow;
    c1=cos(rotz);
    s1=sin(rotz);
    c2=cos(roty);
    s2=sin(roty);
    c3=cos(rotx);
    s3=sin(rotx);
  
    a11=c1*c2/scal_x;
    a12=c2*s1/scal_x;
    a13=s2/scal_x;
    a21=-(c3*s1+c1*s2*s3)/scal_y;
    a22=(c1*c3-s1*s2*s3)/scal_y;
    a23=c2*s3/scal_y;
    a31=(s1*s3-c1*c3*s2)/scal_z;
    a32=-(c3*s1*s2+c1*s3)/scal_z;
    a33=c2*c3/scal_z;
    trigoff=-1;
    for (philint=0;philint<Samples[1];philint++)
      for (thetalint=0;thetalint<Samples[0];thetalint++)
      {
        trigoff++;

        tau_x=ar_tau_x[trigoff];
        tau_y=ar_tau_y[trigoff];
        tau_z=ar_tau_z[trigoff];
        
        alpha_x=ar_alpha_x[trigoff];
        alpha_y=ar_alpha_y[trigoff];
        alpha_z=ar_alpha_z[trigoff];
        
        beta_x=ar_beta_x[trigoff];
        beta_y=ar_beta_y[trigoff];
        beta_z=ar_beta_z[trigoff];

        ur=alpha_x*off_x0+alpha_y*off_y0+alpha_z*off_z0;
        vr=beta_x*off_x0+beta_y*off_y0+beta_z*off_z0;      

        ntaux=a11*tau_x+a12*tau_y+a13*tau_z;
        ntauy=a21*tau_x+a22*tau_y+a23*tau_z;
        ntauz=a31*tau_x+a32*tau_y+a33*tau_z;

        gamma_value=sqrt(ntaux*ntaux+ntauy*ntauy+ntauz*ntauz);

        GetSphericalAngles(&ntheta,&nphi,ntaux,ntauy,ntauz);

        ncostheta=cos(ntheta);
        nsintheta=sin(ntheta);
        ncosphi=cos(nphi);
        nsinphi=sin(nphi);
        
        nbeta_x=-ncostheta*nsinphi;
        nbeta_y=-nsintheta*nsinphi;
        nbeta_z=ncosphi;
        
        nalpha_x=-nsintheta;
        nalpha_y=ncostheta;
        nalpha_z=0;
        
        for (ulint=0;ulint<Samples[2];ulint++)
        {
          Off=thetalint+Samples[0]*(philint+Samples[1]*ulint);
          u=MyTrig->u[ulint]-ur;
          for (vlint=0;vlint<Samples[3];vlint++)
          {
            v=MyTrig->v[vlint]-vr;  
            
            aubv_x=alpha_x*u+beta_x*v;
            aubv_y=alpha_y*u+beta_y*v;
            aubv_z=alpha_z*u+beta_z*v;
            
            Aaubv_x=a11*aubv_x+a12*aubv_y+a13*aubv_z;
            Aaubv_y=a21*aubv_x+a22*aubv_y+a23*aubv_z;
            Aaubv_z=a31*aubv_x+a32*aubv_y+a33*aubv_z;
            
            newu=nalpha_x*Aaubv_x+nalpha_y*Aaubv_y+nalpha_z*Aaubv_z;
            newv=nbeta_x*Aaubv_x+nbeta_y*Aaubv_y+nbeta_z*Aaubv_z;
            
            a=newu*newu+newv*newv;
            switch (shapetype)
            {
              case 1 :
                if (a<1)
                  My4D->Signal[Off+Mult*vlint]+=mypow*2*sqrt(1.0-a)/gamma_value;
                break;
              case 2 :
                My4D->Signal[Off+Mult*vlint]+=mypow*sqrtpi*exp(-a)/gamma_value;
                break;
              default :
                Error("Function of wrong type detected\n");
            }
          }
        }
      }
  }
  Print(_DNormal,"MakeSinogram: Done              \n");
  Free(Samples);
  return My4D;
}


/**************************************************************************
[NAME]
main

[SYNOPSIS]
void main(void)

[DESCRIPTION]
The main program.

[REVISION]
6/6-95 Peter Toft 
**************************************************************************/
void main(int argc,char * argv[])
{
  int RealTid1,RealTid2;
  float Tid;
  char Value[100];

  GetDateTime(Value,_RealTime);
  sscanf(Value,"%i",&RealTid1);
  Tid=clock();
  
  if (argc!=2)
    Error("Usage -> '3D_RadonAna inifile' to generate phantom\n");

  IniBuffer=ReadIni(argv[1]);      /* Read INI file into IniBuffer   */
  
  init();

  if (GenerateVolume==1)
  {
    Print(_DNormal,"Image generation begin\n");
    Volume=MakeVolume();
    InfoMultiStruct(_DNormal,Volume);
    WriteMult(Volume,_F3F,_Signal);
    Print(_DNormal,"Image generation done\n");
    FreeMultiStruct(Volume);
  }
  if (GenerateSinogram==1)
  {
    Print(_DNormal,"Radon generation begin\n");
    Sinogram=MakeSinogram();
    InfoMultiStruct(_DNormal,Sinogram);
    WriteMult(Sinogram,_F4F,_Signal);
    FreeMultiStruct(Sinogram);
    Print(_DNormal,"Radon generation done\n");
  }
  Print(_DNormal,"Writing file\n\n");

  GetDateTime(Value,_RealTime);
  sscanf(Value,"%i",&RealTid2);
  Tid=(clock()-Tid)/(float)CLOCKS_PER_SEC;
  Print(_DNormal,"-------------------------------------------------\n");
  Print(_DNormal,"3D_RadonAna: Program was active for %.2f seconds\n",Tid) ;
  Print(_DNormal,"             World time elapsed %d seconds\n",
	(RealTid2-RealTid1));
  Print(_DNormal,"             Program used %.2f %% cpu time\n",
         (Tid>((float)RealTid2-RealTid1))?99.99:(Tid/((float)RealTid2-RealTid1)*100));
  Print(_DNormal,"-------------------------------------------------\n");
  CloseLog();
}








