/*****************************************************************
[HEADER]
  
  This toolbox contains functions for reconstruction from 3D line integrals.
  The toolbox uses to a large extend {\tt MultiStruct} defined in 
  the file {\tt 3dtools.c}.
  
  
{\tt TrigDef}
{\small
\begin{verbatim}
typedef struct {
  float *u;
  float *v;
  float *theta;
  float *phi;
  int   ThetaSamples;
  int   PhiSamples;
  int   USamples;
  int   VSamples;
  float Thetamin;
  float Phimin;
  float Umin;
  float Vmin;
  float DeltaTheta;
  float DeltaPhi;
  float DeltaU;
  float DeltaV;
  float *tau_x;
  float *tau_y;
  float *tau_z;
  float *alpha_x;  
  float *alpha_y;  
  float *alpha_z;  
  float *beta_x;  
  float *beta_y;  
  float *beta_z;  
} TrigDef;
\end{verbatim}}
\vsp

The directional vectors are indexed as 

{\tt alpha\_x[thetaint+phiint*ThetaSamples]=alpha\_x(thetaint,phi)}

\vsp

{\large XYZDef}
{\small
\begin{verbatim}
typedef struct {
  float *x;
  float *y;
  float *z;
  int  XSamples;
  int  YSamples;
  int  ZSamples;
  float Xmin;
  float Ymin;
  float Zmin;
  float DeltaX;
  float DeltaY;
  float DeltaZ;
} XYZDef;
\end{verbatim}}
\vsp


Peter Toft and Peter Philipsen

Electronics Institute, Building 349,

Technical University of Denmark,

2800 Lyngby

Denmark

ptoft,pap@ei.dtu.dk
*****************************************************************/
#include <math.h>
#include <malloc.h>
#include "3dtools.h"
#include "3dadds.h"
#include "3dcalc.h"
#include "3dmisc.h"



/****************************************************************
[NAME]
GenerateTrigDef

[SYNOPSIS]
 TrigDef * GenerateTrigDef(int ThetaSamples,
                           int PhiSamples,
                           int USamples,
                           int VSamples,
                           float Thetamin,
                           float Phimin,
                           float Umin,
                           float Vmin,
                           float DeltaTheta,
                           float DeltaPhi,
                           float DeltaU,
                           float DeltaV)

[DESCRIPTION]
This function returnes a {\tt TrigDef} with proper trigonometric 
arrays defined. The structure is used for 3D reconstruction from 
line integrals.

[USAGE]
{\tt MyTrigDef=GenerateTrigDef(ThetaSamples,PhiSamples,USamples,VSamples,
			       Thetamin,Phimin,Umin,Vmin,
			       DeltaTheta,DeltaPhi,DeltaU,DeltaV);}

[REVISION]
20/5-95 Peter Toft\\
20/11-95 Peter Toft - Corrected Phimin and Thetamin entry (not set).\\
10/12-95 Peter Toft - Major change. Dropped OrthoVecors and made 
individual vectors.
****************************************************************/
TrigDef * GenerateTrigDef(int ThetaSamples,
			  int PhiSamples,
			  int USamples,
			  int VSamples,
			  float Thetamin,
			  float Phimin,
			  float Umin,
			  float Vmin,
			  float DeltaTheta,
			  float DeltaPhi,
			  float DeltaU,
			  float DeltaV)
{
  TrigDef *MyTrig;
  int i,j,off,ThetaPhiSamples;
  float theta,phi,costheta,sintheta,cosphi,sinphi;
  float *costhetaar,*sinthetaar;

  if(!(MyTrig=(TrigDef *) malloc(sizeof(TrigDef)))) 
    Error( "Memory allocation problems (GenerateTrigDef)" );
  
  MyTrig->ThetaSamples=ThetaSamples;
  MyTrig->PhiSamples=PhiSamples;
  MyTrig->USamples=USamples;
  MyTrig->VSamples=VSamples;
  MyTrig->Umin=Umin;
  MyTrig->Vmin=Vmin;
  MyTrig->Phimin=Phimin;
  MyTrig->Thetamin=Thetamin;
  MyTrig->DeltaTheta=DeltaTheta;
  MyTrig->DeltaPhi=DeltaPhi;
  MyTrig->DeltaU=DeltaU;
  MyTrig->DeltaV=DeltaV;
  MyTrig->u=FloatVector(USamples);
  MyTrig->v=FloatVector(VSamples);
  MyTrig->theta=FloatVector(ThetaSamples);
  MyTrig->phi=FloatVector(PhiSamples);

  costhetaar=FloatVector(ThetaSamples);
  sinthetaar=FloatVector(ThetaSamples);
  for (i=0;i<ThetaSamples;i++) 
  {
    theta=MyTrig->theta[i]=Thetamin+i*DeltaTheta;
    costhetaar[i]=cos(theta);
    sinthetaar[i]=sin(theta);
  }

  for (i=0;i<USamples;i++) MyTrig->u[i]=Umin+i*DeltaU;
  for (i=0;i<VSamples;i++) MyTrig->v[i]=Vmin+i*DeltaV;
  for (i=0;i<PhiSamples;i++) MyTrig->phi[i]=Phimin+i*DeltaPhi;

  ThetaPhiSamples=ThetaSamples*PhiSamples;
    
  MyTrig->tau_x=FloatVector(ThetaPhiSamples);
  MyTrig->tau_y=FloatVector(ThetaPhiSamples);
  MyTrig->tau_z=FloatVector(ThetaPhiSamples);
  MyTrig->alpha_x=FloatVector(ThetaPhiSamples);
  MyTrig->alpha_y=FloatVector(ThetaPhiSamples);
  MyTrig->alpha_z=FloatVector(ThetaPhiSamples);
  MyTrig->beta_x=FloatVector(ThetaPhiSamples);
  MyTrig->beta_y=FloatVector(ThetaPhiSamples);
  MyTrig->beta_z=FloatVector(ThetaPhiSamples);

  off=0;
  for (j=0;j<PhiSamples;j++)
  {
    phi=MyTrig->phi[j];
    cosphi=cos(phi);
    sinphi=sin(phi);
    for (i=0;i<ThetaSamples;i++)
    {
      costheta=costhetaar[i];
      sintheta=sinthetaar[i];
      MyTrig->tau_x[off]=costheta*cosphi;              /* tau_x   */
      MyTrig->tau_y[off]=sintheta*cosphi;              /* tau_y   */
      MyTrig->tau_z[off]=sinphi;                       /* tau_z   */
      MyTrig->alpha_x[off]=-sintheta;                  /* alpha_x */
      MyTrig->alpha_y[off]=costheta;                   /* alpha_y */
      MyTrig->alpha_z[off]=0.0;                        /* alpha_z */
      MyTrig->beta_x[off]=-costheta*sinphi;            /* beta_x  */
      MyTrig->beta_y[off]=-sintheta*sinphi;            /* beta_y  */     
      MyTrig->beta_z[off++]=cosphi;                    /* beta_z  */
    }
  }


  Free(costhetaar);
  Free(sinthetaar);
  return MyTrig;
} 


/****************************************************************
[NAME]
GenerateXYZ

[SYNOPSIS]
 XYZDef * GenerateXYZ(int XSamples,
                      int YSamples,
                      int ZSamples,
                      float Xmin,
                      float Ymin,
                      float Zmin,
                      float DeltaX,
                      float DeltaY,
                      float DeltaZ)


[DESCRIPTION]

This function returnes a {\tt XZYZDef} with proper arrays defined for
sampling the volume domain. The structure is used for 3D
reconstruction from line integrals.

[USAGE]
{\tt MyXYZDef=GenerateXYZ(XSamples,YSamples,ZSamples,
		          Xmin,Ymin,Zmin,
		          DeltaX,DeltaY,DeltaZ);}

[REVISION]
20/5-95 Peter Toft
****************************************************************/
XYZDef * GenerateXYZ(int XSamples,
		     int YSamples,
		     int ZSamples,
		     float Xmin,
		     float Ymin,
		     float Zmin,
		     float DeltaX,
		     float DeltaY,
		     float DeltaZ)
{
  int i;
  XYZDef *MyXYZ;

  if(!(MyXYZ=(XYZDef *) malloc(sizeof(XYZDef)))) 
    Error( "Memory allocation problems (GenerateXYZ)" );
  
  MyXYZ->XSamples=XSamples;
  MyXYZ->YSamples=YSamples;
  MyXYZ->ZSamples=ZSamples;
  MyXYZ->Xmin=Xmin;
  MyXYZ->Ymin=Ymin;
  MyXYZ->Zmin=Zmin;
  MyXYZ->DeltaX=DeltaX;
  MyXYZ->DeltaY=DeltaY;
  MyXYZ->DeltaZ=DeltaZ;
  MyXYZ->x=FloatVector(XSamples);
  MyXYZ->y=FloatVector(YSamples);
  MyXYZ->z=FloatVector(ZSamples);

  for (i=0;i<XSamples;i++) MyXYZ->x[i]=Xmin+i*DeltaX;
  for (i=0;i<YSamples;i++) MyXYZ->y[i]=Ymin+i*DeltaY;
  for (i=0;i<ZSamples;i++) MyXYZ->z[i]=Zmin+i*DeltaZ;

  return MyXYZ;
}


/****************************************************************
[NAME]
GenerateXYZ\_fromMultiStruct

[SYNOPSIS]
 XYZDef *GenerateXYZ_fromMultiStruct(MultiStruct *MyMultiStruct)

[DESCRIPTION]

The function generates the {\tt XYZDef} from a three dimensional 
{\tt MultiStruct}.

[USAGE]
{\tt MyXYZ=GenerateXYZ\_fromMultiStruct(MyMultiStruct);}

[NOTE]
See {\tt GenerateXYZ}.

[REVISION]
20/5-95 Peter Toft
****************************************************************/
XYZDef *GenerateXYZ_fromMultiStruct(MultiStruct *MyMultiStruct)
{
  int *Samples;
  float *Min,*Delta;

  Samples=MyMultiStruct->Samples;
  Delta=MyMultiStruct->Delta;
  Min=MyMultiStruct->Min;

  if (MyMultiStruct->Dim!=3) 
    Error("Non-3D MultiStruct (GenerateXYZ_fromMultiStruct)\n");

  return GenerateXYZ(Samples[0],Samples[1],Samples[2],
		     Min[0],Min[1],Min[2],
		     Delta[0],Delta[1],Delta[2]);
}


/****************************************************************
[NAME]
GenerateXYZ\_fromIniFile

[SYNOPSIS]
 XYZDef *GenerateXYZ_fromIniFile()

[DESCRIPTION]

The function generates the {\tt XYZDef} from data from the IniFile


[USAGE]
{\tt MyXYZ=GenerateXYZ\_fromIniFile();}

[NOTE]
See {\tt GenerateXYZ}

[REVISION]
20/5-95 Peter Toft
****************************************************************/
XYZDef *GenerateXYZ_fromIniFile(void)
{
  return GenerateXYZ(IniFile3D.XSamples,
                     IniFile3D.YSamples,
                     IniFile3D.ZSamples,
                     IniFile3D.Xmin,
                     IniFile3D.Ymin,
                     IniFile3D.Zmin,
                     IniFile3D.DeltaX,
                     IniFile3D.DeltaY,
                     IniFile3D.DeltaZ);
}


/****************************************************************
[NAME]
GenerateTrigDef\_fromMultiStruct

[SYNOPSIS]
 TrigDef *GenerateTrigDef_fromMultiStruct(MultiStruct *MyMultiStruct)

[DESCRIPTION]

The function generates the {\tt TrigDef} from a four dimensional 
{\tt MultiStruct}.

[USAGE]
{\tt MyTrig=GenerateTrigDef\_fromMultiStruct(MyMultiStruct);}

[NOTE]
See {\tt GenerateTrigDef}.

[REVISION]
20/5-95 Peter Toft
****************************************************************/
TrigDef *GenerateTrigDef_fromMultiStruct(MultiStruct *MyMultiStruct)
{
  int *Samples;
  float *Min,*Delta;

  Samples=MyMultiStruct->Samples;
  Delta=MyMultiStruct->Delta;
  Min=MyMultiStruct->Min;

  if (MyMultiStruct->Dim!=4) 
    Error("Non-4D MultiStruct (GenerateTrigDef_fromMultiStruct)\n");

  return GenerateTrigDef(Samples[0],
                         Samples[1],
                         Samples[2],
                         Samples[3],
			 Min[0],
                         Min[1],
                         Min[2],
                         Min[3],
			 Delta[0],
                         Delta[1],
                         Delta[2],
                         Delta[3]);
}


















