#include <math.h>
#include <malloc.h>
#include "3dtools.h"
#include "3dadds.h"
#include "3dcalc.h"
#include "3dmisc.h"
#include "3dfilter.h"

/* Define SGI if you want to use the pragma settings for SGI parallel
   compiler */

/****************************************************************
[NAME]
WindowVolume

[SYNOPSIS]
 void WindowVolume(MultiStruct *My3D,int width)

[DESCRIPTION] 
The function windows a volume with a Hann window in all
three directions. The parameter {\tt width} determines 
how many samples are altered measured from the edges.

[USAGE]
{\tt WindowVolume(InVolume,5);}

[NOTE]
Definition may be altered in the future.

[REVISION]
20/5-95 Peter Toft\\
10/8-95 Peter Toft (Variable width of window). 
****************************************************************/
void WindowVolume(MultiStruct *My3D,int width)
{
  int ii,xint,yint,zint;
  int XSamples,YSamples,ZSamples;
  float wx,wy,weight;
  float *weightvector;
  float *Signal;

  Signal=My3D->Signal;
  weightvector=FloatVector(width);
  XSamples=My3D->Samples[0];
  YSamples=My3D->Samples[1];
  ZSamples=My3D->Samples[2];

  for (ii=0;ii<width;ii++)
    weightvector[ii]=0.5*(1.0-cos((M_PI*ii)/width));
  
  for (xint=0;xint<width;xint++)
  {
    wx=weightvector[xint];
    for (yint=0;yint<width;yint++)
    {
      wy=weightvector[yint];
      for (zint=0;zint<width;zint++)
      {
        weight=wx*wy*weightvector[zint];
        Signal[xint
               +XSamples*(yint+zint*YSamples)]*=weight;
        Signal[xint
               +XSamples*(yint+(ZSamples-1-zint)*YSamples)]*=weight;
        Signal[xint
               +XSamples*(YSamples-1-yint+(ZSamples-1-zint)*YSamples)]*=weight;
        Signal[xint
               +XSamples*(YSamples-1-yint+zint*YSamples)]*=weight;
        Signal[XSamples-1-xint
               +XSamples*(yint+zint*YSamples)]*=weight;
        Signal[XSamples-1-xint
               +XSamples*(yint+(ZSamples-1-zint)*YSamples)]*=weight;
        Signal[XSamples-1-xint
               +XSamples*(YSamples-1-yint+(ZSamples-1-zint)*YSamples)]*=weight;
        Signal[XSamples-1-xint
               +XSamples*(YSamples-1-yint+zint*YSamples)]*=weight;
      }
    }
  }
  Free(weightvector);
}


/****************************************************************
[NAME]
CutRadial

[SYNOPSIS]
 void CutRadial(MultiStruct *My3D,float reldist)

[DESCRIPTION] 
The function a volume radially from the corners, so the voxel 
values are zero when outside a ball with radius {\tt reldist}
times the smallest distance from the centre to the edge, i.e., 
{\tt reldist=1} implies that the ball touches the closest edge
and {\tt reldist}$\ge\sqrt{3}$ has no effect.

[USAGE]
{\tt WindowRadial(InVolume,1.0);}

[NOTE]
Definition may be altered in the future.

[REVISION]
23/11-95 Peter Toft
****************************************************************/
void WindowRadial(MultiStruct *My3D,float reldist)
{
  int xint,yint,zint;
  int off,XSamples,YSamples,ZSamples;
  float tmpf,tmpf2,lim;
  float *Signal;
  float *xf2,*yf2,*zf2;

  Print(_DDebug,"Windowing MultiStruct %f (WindowRadial)\n",reldist);

  if (My3D->Dim!=3)
    Error("The MultiStruct should be 3D (WindowRadial)");

  XSamples=My3D->Samples[0];
  YSamples=My3D->Samples[1];
  ZSamples=My3D->Samples[2];

  /* The following restrictions might be dropped in the future */
  if ((XSamples!=YSamples) || (YSamples!=ZSamples))
    Error("The MultiStruct should be cubic (WindowRadial)");
  if ((My3D->Delta[0]!=My3D->Delta[1]) || (My3D->Delta[0]!=My3D->Delta[2]))
    Error("The MultiStruct should have equal samling parms (WindowRadial)");
  if ((My3D->Min[0]!=My3D->Min[1]) || (My3D->Min[0]!=My3D->Min[2]))
    Error("The MultiStruct should have equal samling parms (WindowRadial)");
 
  Signal=My3D->Signal;

  xf2=FloatVector(XSamples);
  yf2=FloatVector(YSamples);
  zf2=FloatVector(ZSamples);

  /* Following can be reduced by 2/3 if only cubic multistructs are used */
  for (xint=0;xint<XSamples;xint++)
  {
    tmpf=xint*My3D->Delta[0]+My3D->Min[0];
    xf2[xint]=tmpf*tmpf;
  }
  for (yint=0;yint<YSamples;yint++)
  {
    tmpf=yint*My3D->Delta[1]+My3D->Min[1];
    yf2[yint]=tmpf*tmpf;
  }
  for (zint=0;zint<ZSamples;zint++)
  {
    tmpf=zint*My3D->Delta[2]+My3D->Min[2];
    zf2[zint]=tmpf*tmpf;
  }
  
  /* lim is set to the squared distrance from (0,0,0) to the edge 
     times the reldist-parameter.                                 */

  lim=reldist*(XSamples-1)*0.5*My3D->Delta[0];
  lim*=lim;

  for (zint=0;zint<ZSamples;zint++)
  {
    tmpf2=zf2[zint];
    for (yint=0;yint<YSamples;yint++)
    {
      tmpf=lim-(tmpf2+yf2[yint]);
      off=XSamples*(yint+zint*YSamples);
      for (xint=0;xint<XSamples;xint++)
        if (xf2[xint]>tmpf) Signal[off+xint]=0.0;
    }
  }
}


/****************************************************************
[NAME]
Filter4DSinogram

[SYNOPSIS]
 void Filter4DSinogram(MultiStruct *My4D,
                       TrigDef *MyTrig,
                       float PhiMin,
                       int WindowParam,
                       float Relative_filtercutoff)

[DESCRIPTION]
The function will apply a highpass filter to the images (u,v)
for each $(\theta,\phi)$. The parameter {\tt PhiMin} determines the 
limiting $\psi$-angle in the geometry $\Omega_{\psi}$. The parameter 
{\tt WindowParam} controls the windowing.
If set to 0 no windowing is done. 
If set to 1 pre-windowing is done. 
A cubic Hann-window is used to reduce the edge artifacts.

[USAGE]
{\tt Filter4DSinogram(My4D,MyTrig,PhiMin,0);}

[NOTE]
Syntax may be changed in the future.

[REVISION]
20/5-95 Peter Toft\\
10/9-95 Peter Toft. Major revision. Made filter for limited 
        angular coverage and made windowing optional.\\
10/12-95 Peter Toft - Dropped OrthoVecors and made 
use of individual vectors. See TrigDef.\\
Dec 13 95 Peter Toft - Use NdimFFT\\
Oct 28 96 Peter Toft remove unused vars\\
May 11 97 Peter Toft Removed garbage and added Hann filter 
weighting for Kelly. Use Select\_Par\_1=1 in the ini-file to 
invoke Hann weighting of the "ramp" filter, and set Direct\_Par\_1
to a value less than one to get faster truncation of the filter. The 
function has an additional argument.
****************************************************************/
void Filter4DSinogram(MultiStruct *My4D,
                      TrigDef *MyTrig,
                      float psi,
                      int WindowParam, 
                      float Relative_filtercutoff)
{
  int i,j,l,m,uuint,vvint,off,Mult;
  int USamples,VSamples,Ur2Samples,Vr2Samples,ThetaSamples,PhiSamples;
  int *Radix2Samples;
  int *Radix2Sampleslocal,id,myid;
  float *InSignal,*UVSignal,*Hann_u_Window,*Hann_v_Window;
  int *Position;
  float *nuuar,*nuvar;
  float InvPI,filter,ff,nuv,nuu,nu_2,nu,ku,kv;
  float alpha_x,alpha_y,alpha_z,nux,nuy,nuz,nux_2,nuy_2,nuz_2;
  float beta_x,beta_y,beta_z,sinpsi,invPI,nuxy;
  float *ar_alpha_x,*ar_alpha_y,*ar_alpha_z;
  float *ar_beta_x,*ar_beta_y,*ar_beta_z;
  float nu_max,cutoff_freq;
 
  if (My4D->Dim!=4)
    Error("Wrong dimension of MultiStruct (Filter4DSinogram)");
  if (My4D->ArrayType!=_RealArray)
    Error("Wrong Array Type. Must be RealArray (Filter4DSinogram)");

  if ((WindowParam!=0) && (WindowParam!=1))
    Error("WindowParam wrong = %i (Filter4DSinogram)\n",WindowParam);

  id=0;

#ifdef SGI
  #pragma parallel
  #pragma local(thetaint,phiint,i,j,l,m,uuint,vvint,off,Mult)
  #pragma local(USamples,VSamples,Ur2Samples,Vr2Samples,ThetaSamples,PhiSamples)
  #pragma local(Radix2Samples,Radix2Sampleslocal,myid)
  #pragma local(InSignal,UVSignal,Hann_u_Window,Hann_v_Window,Position,nuuar,nuvar)
  #pragma local(InvPI,filter,ff,nuv,nuu,nu_2,nu,ku,kv)
  #pragma local(alpha_x,alpha_y,alpha_z,nux,nuy,nuz,nux_2,nuy_2,nuz_2)
  #pragma local(beta_x,beta_y,beta_z,sinpsi,invPI,nuxy)
  #pragma local(ar_alpha_x,ar_alpha_y,ar_alpha_z)
  #pragma local(ar_beta_x,ar_beta_y,ar_beta_z)
  #pragma shared(id)
  #pragma byvalue(My4D,MyTrig,psi,WindowParam)
#endif

  {
  
    invPI=1.0/M_PI;    
    InSignal=My4D->Signal;
    Position=IntVector(2);
    ThetaSamples=My4D->Samples[0];
    PhiSamples=My4D->Samples[1];
    USamples=My4D->Samples[2];
    VSamples=My4D->Samples[3];
    Radix2Samples=IntVector(2);
    Ur2Samples=Radix2Samples[0]=NextRadix2(USamples);
    Vr2Samples=Radix2Samples[1]=NextRadix2(VSamples);
    Hann_u_Window=FloatVector(USamples);
    Hann_v_Window=FloatVector(VSamples);
    Position[0]=0;
    Position[1]=0;
    
    sinpsi=sin(psi);
    
    ar_alpha_x=MyTrig->alpha_x;
    ar_alpha_y=MyTrig->alpha_y;
    ar_alpha_z=MyTrig->alpha_z;
    ar_beta_x=MyTrig->beta_x;
    ar_beta_y=MyTrig->beta_y;
    ar_beta_z=MyTrig->beta_z;
    
    ff=PI*2.0/(USamples-1);
    for (i=0;i<USamples;i++)
      Hann_u_Window[i]=0.5*(1-cos(ff*i));
    ff=PI*2.0/(VSamples-1);
    for (i=0;i<VSamples;i++)
      Hann_v_Window[i]=0.5*(1-cos(ff*i));
    
    
    InvPI=1.0/(PI);
    ku=1.0/(Ur2Samples*My4D->Delta[2]);
    kv=1.0/(Vr2Samples*My4D->Delta[3]);
    
    nuuar=FloatVector(Ur2Samples);
    nuvar=FloatVector(Vr2Samples);
    
    for (i=0;i<Ur2Samples/2;i++) nuuar[i]=i*ku;
    for (i=Ur2Samples/2;i<Ur2Samples;i++) nuuar[i]=(Ur2Samples-i)*ku;
    for (j=0;j<Vr2Samples/2;j++) nuvar[j]=j*kv;
    for (j=Vr2Samples/2;j<Vr2Samples;j++) nuvar[j]=(Vr2Samples-j)*kv;
    
    Mult=ThetaSamples*PhiSamples;
    nu_max=0.5*sqrt((Ur2Samples*ku)*(Ur2Samples*ku)+
                    (Vr2Samples*kv)*(Vr2Samples*kv));
    cutoff_freq=nu_max*Relative_filtercutoff;

#ifdef SGI
#pragma critical
#endif
    {
      myid=id;
    }
    
    Print(_DDebug,"Allocating memory for UVSignal (Filter4DSinogram)\n");
    UVSignal=FloatVector(Ur2Samples*Vr2Samples*_ComplexArray);
    
    Radix2Sampleslocal=IntVector(2);
    Radix2Sampleslocal[0]=Radix2Samples[0];
    Radix2Sampleslocal[1]=Radix2Samples[1];
    
#ifdef SGI
#pragma pfor iterate(off=0;Mult;1)
#pragma schedtype(simple)
#endif

    for (off=0;off<Mult;off++)
      {
        Print(_DNormal,"Sinogram filtering: %3i (%i) done\r",
              off,Mult);
      
        alpha_x=ar_alpha_x[off];
        alpha_y=ar_alpha_y[off];
        alpha_z=ar_alpha_z[off];
        beta_x=ar_beta_x[off];
        beta_y=ar_beta_y[off];
        beta_z=ar_beta_z[off];

        l=0;
        for (vvint=0;vvint<VSamples;vvint++) {
	  ff=Hann_v_Window[vvint];
	  for (uuint=0;uuint<USamples;uuint++) {
	    if (WindowParam==1)
	      UVSignal[l++]=ff*Hann_u_Window[uuint]*
		InSignal[off+Mult*(uuint+USamples*vvint)];
	    else
	      UVSignal[l++]=InSignal[off+Mult*(uuint+USamples*vvint)];
	    UVSignal[l++]=0.0;
	  }
	  for (uuint=USamples;uuint<Ur2Samples;uuint++) {
	    UVSignal[l++]=0.0;
                UVSignal[l++]=0.0;
	  }
	}
        for (vvint=VSamples;vvint<Vr2Samples;vvint++) {
          for (uuint=0;uuint<Ur2Samples;uuint++) {
	    UVSignal[l++]=0.0;
	    UVSignal[l++]=0.0;
	  }
	}
	
        NdimFFT(UVSignal,Radix2Sampleslocal,2,_FFT);      

        m=0;
        for (vvint=0;vvint<Vr2Samples;vvint++) {
	  nuv=nuvar[vvint];
	  for (uuint=0;uuint<Ur2Samples;uuint++) {
	    nuu=nuuar[uuint];
	    nux=nuu*alpha_x+nuv*beta_x;
	    nuy=nuu*alpha_y+nuv*beta_y;
	    nuz=nuu*alpha_z+nuv*beta_z;
	    nux_2=nux*nux;
	    nuy_2=nuy*nuy;
	    nuz_2=nuz*nuz;
	    nu_2=nux_2+nuy_2+nuz_2;
	    nuxy=sqrt(nux_2+nuy_2);
	    nu=sqrt(nuu*nuu+nuv*nuv);
	    if (nuxy<=nu*sinpsi) {
	      filter=invPI*nu;
	    } else {
	      filter=nu/(2.0*asin(nu*sinpsi/nuxy));
	    }
	    
	    if (Relative_filtercutoff>1e-15) {
	      if (fabs(nu)>cutoff_freq)
		filter*=0.0;
	      else
		filter*=0.5*(1+cos(nu/cutoff_freq));
	    }
	    
	    UVSignal[m++]*=filter; /* Multiply filter on Real part */
	    UVSignal[m++]*=filter; /* Multiply filter on Imag part */
	  }
	}
        NdimFFT(UVSignal,Radix2Sampleslocal,2,_IFFT);      
      
        for (vvint=0;vvint<VSamples;vvint++) {
	  l=vvint*Ur2Samples*_ComplexArray;
	  for (uuint=0;uuint<USamples;uuint++) {
	    /* Taking real part out and dropping imag part=0 */
	    InSignal[off+Mult*(uuint+USamples*vvint)]=UVSignal[l++]; 
	    l++;
	  }
	}
      }
    Free(UVSignal);
  }/* END parallel*/
  
  Print(_DNormal,"Done      (Filter4DSinogram)             \n");
}


/****************************************************************
[NAME]
FilterVolume

[SYNOPSIS]
 void FilterVolume(MultiStruct *My3D,float psi)

[DESCRIPTION] 
The function will filter a given volume with a high pass
filter.  The function is intended for a filtering after backprojecting
algorithm.

[USAGE]
{\tt FilterVolume(InVolume,-MySinogram->psi\_min);}

[NOTE]
The syntax of this function might change in the future.

[REVISION]
20/5-95 Peter Toft.\\
6/6-95 Peter Philipsen - Filter norm.\\
6/9-95 Peter Toft  - Made filter in geometry $\Omega_\psi$ 
       (See Clack/Colsher). Appended limit angle.\\
12/9-95 Peter Toft Corrections in normalization.\\
Dec 13 Peter Toft - Use NdimFFT.
****************************************************************/
void FilterVolume(MultiStruct *My3D,float psi)
{
  int i,j,k,l;
  int XSamples,YSamples,ZSamples;
  int Samples[3];
  float *Signal;
  float *fxar,*fyar,*fzar,fx,fy,fz;
  float foff,filter,sqf,sinpsi,nu,nuxy,InvPI;

  sinpsi=sin(psi);
  if (My3D->Dim!=3)
    Error("Wrong dimension of MultiStruct (FilterVolume)");
  if (My3D->ArrayType!=_ComplexArray)
    Error("Wrong Array Type. Must be ComplexArray (FilterVolume)");

  Signal=My3D->Signal;
  
  XSamples=Samples[0]=My3D->Samples[0];
  YSamples=Samples[1]=My3D->Samples[1];
  ZSamples=Samples[2]=My3D->Samples[2];

  if (XSamples!=NextRadix2(XSamples))
    Error("Samples[0] must be a power of two\n");

  if (YSamples!=NextRadix2(YSamples))
    Error("Samples[1] must be a power of two\n");

  if (ZSamples!=NextRadix2(ZSamples))
    Error("Samples[2] must be a power of two\n");

  InvPI=1.0/(M_PI);
  fx=1.0/(XSamples*My3D->Delta[0]);
  fy=1.0/(YSamples*My3D->Delta[1]);
  fz=1.0/(ZSamples*My3D->Delta[2]);

  Print(_DNormal,"Forward FFT\n");

  NdimFFT(Signal,Samples,3,_FFT);

  Print(_DNormal,"Filtering.\n");

  fxar=FloatVector(XSamples);
  fyar=FloatVector(YSamples);
  fzar=FloatVector(ZSamples);

  for (i=0;i<XSamples/2;i++)
  {
    sqf=i*fx;
    fxar[i]=sqf*sqf;
  }
  for (i=XSamples/2;i<XSamples;i++) 
  {
    sqf=(XSamples-i)*fx;
    fxar[i]=sqf*sqf;
  }
  for (j=0;j<YSamples/2;j++)
  {
    sqf=j*fy;
    fyar[j]=sqf*sqf;
  }
  for (j=YSamples/2;j<YSamples;j++)
  {
    sqf=(YSamples-j)*fy;
    fyar[j]=sqf*sqf;
  }
  for (k=0;k<ZSamples/2;k++)
  {
    sqf=k*fz;
    fzar[k]=sqf*sqf;
  }
  for (k=ZSamples/2;k<ZSamples;k++)
  {
    sqf=(ZSamples-k)*fz;
    fzar[k]=sqf*sqf;
  }

  l=0;
  for (k=0;k<ZSamples;k++)
  {	
    Print(_DNormal,"3D Filtering: %i(%i) iterations done\r",i,XSamples);
    fz=fzar[k];
    for (j=0;j<YSamples;j++)
    {
      fy=fyar[j];
      foff=fz+fy;
      for (i=0;i<XSamples;i++)
      {
        fx=fxar[i];
	nuxy=sqrt(fx+fy);
        nu=sqrt(foff+fx);
        if (nuxy<=nu*sinpsi) 
          filter=nu*InvPI;
        else
          filter=nu/(2*asin(nu*sinpsi/nuxy));
	Signal[l++]*=filter;
	Signal[l++]*=filter;
      }
    }
  }
  Free(fxar);
  Free(fyar);
  Free(fzar);
  Print(_DNormal,"3D Filtering: %i(%i) iterations done\n",i,XSamples);
  Print(_DNormal,"Inverse FFT\n");
  NdimFFT(Signal,Samples,3,_IFFT);
  Print(_DNormal,"3D Filtering: Done\n");
  Print(_DNormal,"Done      (FilterVolume)                   \n");
}


/************************************************************************
[NAME]
ComplexNdimFFT

[SYNOSPSIS]
 void ComplexNdimFFT(float *data,
                     unsigned long *nn,
                     int ndim,
                     int isign);

[DESCRIPTION] 

Calculates the n-dimensional Fourier transformation of a n-dimensional
data structure in data. (See Numerical Recipes pp. 523-524 for a
complete description). Data are stored in sequence in {\tt data}. 
The dimensions in {\tt dim}, where {\tt dim[1]} is the size in the 
first direction, and so on\ldots

[USAGE]
{\tt ComplexNdimFFT(\&Test[-1],\&dim[-1],3,\_FFT);}

Calculates the 3-dimensional FFT of the image stored in {\tt data}.

[NOTE]
Be sure that dimensions is powers of two. See also {\tt NewNdimFFT}.

[REVISION]
Oct. 94, JJJ and PT\\
Dec 13 PT - description and ref to NewNdimFFT

***************************************************************************/
void ComplexNdimFFT(float *data,unsigned long *nn, int ndim, int isign)
{
  int idim,count;
  unsigned long i1,i2,i3,i2rev,i3rev,ip1,ip2,ip3,ifp1,ifp2;
  unsigned long ibit,k1,k2,n,nprev,nrem,ntot;
  float tempi,tempr,Temp;
  double theta,wi,wpi,wpr,wr,wtemp;

  for(ntot=1,idim=1;idim<=ndim;idim++)
     ntot *= nn[idim];
  nprev=1;
  for(idim=ndim;idim>=1;idim--) {
     n=nn[idim];
     nrem=ntot/(n*nprev);
     ip1=nprev<<1;
     ip2=ip1*n;
     ip3=ip2*nrem;
     i2rev=1;
     for(i2=1;i2<=ip2;i2+=ip1) {
        if(i2<i2rev) {
          for(i1=i2;i1<=i2+ip1-2;i1+=2) {
             for(i3=i1;i3<=ip3;i3+=ip2) {
                i3rev=i2rev+i3-i2;
                Swap(data[i3],data[i3rev]);
                Swap(data[i3+1],data[i3rev+1]);
	     }
     	  }
	}
        ibit=ip2>>1;
        while(ibit>=ip1 && i2rev>ibit) {
             i2rev-=ibit;
             ibit>>=1;   
	}
        i2rev+=ibit;
     }
     ifp1=ip1;
     while(ifp1<ip2) {
  	ifp2=ifp1<<1;
	theta=isign*6.28318530717959/(ifp2/ip1);
	wpr=cos(theta);
	wpi=sin(theta);
	wr=1.0;
	wi=0.0;
	for(i3=1;i3<=ifp1;i3+=ip1) {
	   for(i1=i3;i1<=i3+ip1-2;i1+=2) {
              for(i2=i1;i2<=ip3;i2+=ifp2) {
		 k1=i2;
		 k2=k1+ifp1;
		 tempr=(float)wr*data[k2]-(float)wi*data[k2+1];
                 tempi=(float)wr*data[k2+1]+(float)wi*data[k2];
		 data[k2]=data[k1]-tempr;
		 data[k2+1]=data[k1+1]-tempi;
		 data[k1] += tempr;
		 data[k1+1] += tempi;
       	      }
	   }
           wr=(wtemp=wr)*wpr-wi*wpi;
           wi=wi*wpr+wtemp*wpi;
	}
      	ifp1=ifp2;
    }
    nprev*=n;
  }

  if (isign==_IFFT) {
    for(count=1; count<=ntot*2; count++) {
      data[count]/=ntot;
    }
  }
}

void ComplexNdimFFT2(float *data,unsigned long *nn, int ndim, int isign)
{
  int idim,count;
  unsigned long i1,i2,i3,i2rev,i3rev,ip1,ip2,ip3,ifp1,ifp2;
  unsigned long ibit,k1,k2,n,nprev,nrem,ntot;
  float tempi,tempr,Temp;
  double theta,wi,wpi,wpr,wr,wtemp;

  for(ntot=1,idim=1;idim<=ndim;idim++)
     ntot *= nn[idim];
  nprev=1;
  for(idim=ndim;idim>=1;idim--) {
     n=nn[idim];
     nrem=ntot/(n*nprev);
     ip1=nprev<<1;
     ip2=ip1*n;
     ip3=ip2*nrem;
     i2rev=1;
     for(i2=1;i2<=ip2;i2+=ip1) {
        if(i2<i2rev) {
          for(i1=i2;i1<=i2+ip1-2;i1+=2) {
             for(i3=i1;i3<=ip3;i3+=ip2) {
                i3rev=i2rev+i3-i2;
                Swap(data[i3],data[i3rev]);
                Swap(data[i3+1],data[i3rev+1]);
	     }
     	  }
	}
        ibit=ip2>>1;
        while(ibit>=ip1 && i2rev>ibit) {
             i2rev-=ibit;
             ibit>>=1;   
	}
        i2rev+=ibit;
     }
     ifp1=ip1;
     while(ifp1<ip2) {
  	ifp2=ifp1<<1;
	theta=isign*6.28318530717959/(ifp2/ip1);
	wpr=cos(theta);
	wpi=sin(theta);
	wr=1.0;
	wi=0.0;
	for(i3=1;i3<=ifp1;i3+=ip1) {
	   for(i1=i3;i1<=i3+ip1-2;i1+=2) {
              for(i2=i1;i2<=ip3;i2+=ifp2) {
		 k1=i2;
		 k2=k1+ifp1;
		 tempr=(float)wr*data[k2]-(float)wi*data[k2+1];
                 tempi=(float)wr*data[k2+1]+(float)wi*data[k2];
		 data[k2]=data[k1]-tempr;
		 data[k2+1]=data[k1+1]-tempi;
		 data[k1] += tempr;
		 data[k1+1] += tempi;
       	      }
	   }
           wr=(wtemp=wr)*wpr-wi*wpi;
           wi=wi*wpr+wtemp*wpi;
	}
      	ifp1=ifp2;
    }
    nprev*=n;
  }

  if (isign==_IFFT) {
    for(count=1; count<=ntot*2; count++) {
      data[count]/=ntot;
    }
  }
}


/****************************************************************
[NAME]
BoxFilterVolume

[SYNOPSIS]
 void BoxFilterVolume(MultiStruct *My3D,int hwidth)

[DESCRIPTION] 
The function filters a volume with a cubic box filter with 
width equal 2*hwidth+1 in all three directions.

[USAGE]
{\tt WindowVolume(InVolume,5);}

[NOTE]
Definition may be altered in the future.

[REVISION]
11/8-95 Peter Toft
****************************************************************/
void BoxFilterVolume(MultiStruct *My3D,int hwidth)
{
  int prod,xint,yint,zint,xii,yii,zii,off;
  int XSamples,YSamples,ZSamples;
  float *FiltSignal,*FiltSignal2;
  float *Signal;
  float sum,norm,totalwidth;
  
  Print(_DNormal,"Filtering (BoxFilterVolume)\n");
  if (My3D->ArrayType!=_RealArray)
    Error("Function cannot use a complex MultiStruct (BoxFilterVolume)\n");
  if (My3D->Dim!=3)
    Error("Function can only use a 3D MultiStruct (BoxFilterVolume)\n");
  if ((My3D->Samples[0]<2*hwidth+1) || 
      (My3D->Samples[1]<2*hwidth+1) || 
      (My3D->Samples[2]<2*hwidth+1))
    Error("MultiStruct is too small %ix%ix%i(BoxFilterVolume)\n",
          My3D->Samples[0],
          My3D->Samples[1],
          My3D->Samples[2]);
  
  Signal=My3D->Signal;
  FiltSignal=FloatVector(My3D->NoOfSamples);
  FiltSignal2=FloatVector(My3D->NoOfSamples);
  
  XSamples=My3D->Samples[0];
  YSamples=My3D->Samples[1];
  ZSamples=My3D->Samples[2];
  
  totalwidth=2*hwidth+1;
  norm=1.0/totalwidth;

  for (zint=0;zint<ZSamples;zint++)
    for (yint=0;yint<YSamples;yint++)
    {
      for (xint=0;xint<hwidth;xint++)
      {
        sum=0.0;
        off=xint+XSamples*(yint+zint*YSamples);
        for (xii=-xint;xii<=hwidth;xii++)
          sum+=Signal[off+xii];
        FiltSignal[off]=sum*norm;
      }
      for (xint=hwidth;xint<XSamples-hwidth;xint++)
      {
        sum=0.0;
        off=xint+XSamples*(yint+zint*YSamples);
        for (xii=-hwidth;xii<=hwidth;xii++)
          sum+=Signal[off+xii];
        FiltSignal[off]=sum*norm;
      }
      for (xint=XSamples-hwidth;xint<XSamples;xint++)
      {
        sum=0.0;
        off=xint+XSamples*(yint+zint*YSamples);
        for (xii=-hwidth;xii<XSamples-xint;xii++)
          sum+=Signal[off+xii];
        FiltSignal[off]=sum*norm;
      }
    } 

  for (zint=0;zint<ZSamples;zint++)
    for (xint=0;xint<XSamples;xint++)
    {
      for (yint=0;yint<hwidth;yint++)
      {
        sum=0.0;
        off=xint+XSamples*(yint+zint*YSamples);
        for (yii=-yint;yii<=hwidth;yii++)
          sum+=FiltSignal[off+yii*XSamples];
        FiltSignal2[off]=sum*norm;
      }
      for (yint=hwidth;yint<YSamples-hwidth;yint++)
      {
        sum=0.0;
        off=xint+XSamples*(yint+zint*YSamples);
        for (yii=-hwidth;yii<=hwidth;yii++)
          sum+=FiltSignal[off+yii*XSamples];
        FiltSignal2[off]=sum*norm;
      }
      for (yint=YSamples-hwidth;yint<YSamples;yint++)
      {
        sum=0.0;
        off=xint+XSamples*(yint+zint*YSamples);
        for (yii=-hwidth;yii<YSamples-yint;yii++)
          sum+=FiltSignal[off+yii*XSamples];
        FiltSignal2[off]=sum*norm;
      }
    } 

  prod=XSamples*YSamples;
  for (xint=0;xint<XSamples;xint++)
    for (yint=0;yint<YSamples;yint++)
    {
      for (zint=0;zint<hwidth;zint++)
      {
        sum=0.0;
        off=xint+XSamples*(yint+zint*YSamples);
        for (zii=-zint;zii<=hwidth;zii++)
          sum+=FiltSignal2[off+zii*prod];
        Signal[off]=sum*norm;
      }
      for (zint=hwidth;zint<ZSamples-hwidth;zint++)
      {
        sum=0.0;
        off=xint+XSamples*(yint+zint*YSamples);
        for (zii=-hwidth;zii<=hwidth;zii++)
          sum+=FiltSignal2[off+zii*prod];
        Signal[off]=sum*norm;
      }
      for (zint=ZSamples-hwidth;zint<ZSamples;zint++)
      {
        sum=0.0;
        off=xint+XSamples*(yint+zint*YSamples);
        for (zii=-hwidth;zii<ZSamples-zint;zii++)
          sum+=FiltSignal2[off+zii*prod];
        Signal[off]=sum*norm;
      }
    } 

  
  Free(FiltSignal);
  Free(FiltSignal2);
  My3D->Signal=Signal;
}


/************************************************************************
[NAME]
NdimFFT

[SYNOSPSIS]
 void NdimFFT(float *data,
              int *nn,
              int ndim,
              int isign);

[DESCRIPTION] 

Calculates the n-dimensional Fourier transformation of a n-dimensional
data structure in data. (See Numerical Recipes pp. 523-524 for a
complete description). Used by {\tt FFTImage}. Data are stored in
sequence in {\tt data}. The dimensions in {\tt dim}, where {\tt dim[0]}
is the size in the first direction, and so on\ldots

This function is meant to give a simpler interface (no old-fortran 
shift problem) to {\tt ComplexNdimFFT}.

[USAGE]
{\tt NdimFFT(Test,dim,3,\_FFT);}

Calculates the 3-dimensional FFT of the image stored in {\tt data}.

[NOTE]
Be sure that dimensions is powers of two. See also {\tt ComplexNdimFFT}.

[REVISION]
Dec 13 95 Peter Toft
***************************************************************************/
void NdimFFT(float *inar,int *samples,int dim,int direction)
{
  ComplexNdimFFT(&inar[-1],(unsigned long *)&samples[-1],dim,direction);
}
void NdimFFT2(float *inar,int *samples,int dim,int direction)
{
  ComplexNdimFFT2(&inar[-1],(unsigned long *)&samples[-1],dim,direction);
}






