#include <math.h>
#include <time.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <malloc.h>

#include "3dtools.h"
#include "3dadds.h"
#include "3dfilter.h"
#include "3dcalc.h"
#include "3dmisc.h"
#include "3dit.h"
#include "Multi_IO.h"

/* Interpolationtype 1=NN 4=BL */

#define NN_Interpolation 1
#define BL_Interpolation 4


/***************************************************************************
[NAME]
Iterative\_ART\_3D

[SYNOPSIS]
 MultiStruct * Iterative_ART_3D(MultiStruct *,XYZDef *,float)

[DESCRIPTION]
This function will estimate the reconstructed volume using ART
in a three dimensional version. The function will return the reconstructed
volume. The input parameters are: The 4D sinogram, a XYZDef defining the 
sampling of the volume and the last parameter determines the number of 
iterations relative to the number of samples in the 4D sinogram. 

[USAGE]
{\tt MultiStruct * Iterative\_ART\_3D(MultiStruct *MyRadon,XYZDef *MyXYZ,
float maxiterations)}

[REVISION]
28/6-95 Peter Toft\\
22/8-95 Peter Toft (Weightfactor on phi - fast hack - central limit theorem)
***************************************************************************/
MultiStruct * Iterative_ART_3D(MultiStruct *MyRadon,
                               XYZDef *MyXYZ,float maxiterations)
{
  TrigDef *MyTrig;
  int TotalIterations,iteration,Interpolation;
  int thetaint,phiint,uuint,vvint,counter,statprints;
  int ThetaSamples,PhiSamples,USamples,VSamples,ii,radonindex,writetext;
  int XSamples,YSamples,ZSamples;
  float A,AAsum,Axsum,fract,procit;
  double decay,beta,alpha;
  VolWeight *MyWeight,*MyWeight2;
  MultiStruct *MyVolume,*OrgVolume;
  int Samples[3];
  float *VolSignal,*RadonSignal;

  Print(_DNormal,"Initializing (Iterative_ART_3D)\n");

  MyTrig=GenerateTrigDef_fromMultiStruct(MyRadon);
  srand(37);

  OrgVolume=ReadMult(IniFile3D.OrgFile,IniFile3D.OrgFileType,_Signal);

  ThetaSamples=MyRadon->Samples[0];
  PhiSamples=MyRadon->Samples[1];
  USamples=MyRadon->Samples[2];
  VSamples=MyRadon->Samples[3];

  XSamples=Samples[0]=MyXYZ->XSamples;
  YSamples=Samples[1]=MyXYZ->YSamples;
  ZSamples=Samples[2]=MyXYZ->ZSamples;

  if (maxiterations<0)
  {
    maxiterations=-maxiterations;
    Interpolation=NN_Interpolation;
  }
  else
    Interpolation=BL_Interpolation;
  
  TotalIterations=(int)(maxiterations*ThetaSamples*
                        PhiSamples*USamples*VSamples);

  statprints=50;
  writetext=TotalIterations/statprints;
  procit=1e2/TotalIterations;

  if (fabs(IniFile3D.Iterative_Par_1-IniFile3D.Iterative_Par_2)<1e-3)
  {
    Print(_DNormal,"Illegal iterative decay parameters (Iterative_ART_3D)\n");
    alpha=beta=1;
  }
  else
  {
    alpha=IniFile3D.Iterative_Par_1;
    beta=
      exp(log(IniFile3D.Iterative_Par_2/IniFile3D.Iterative_Par_1)/statprints);
  }
  decay=alpha;

  MyWeight=(VolWeight *)malloc(Interpolation*sizeof(VolWeight)*
                               max(max(XSamples,YSamples),ZSamples));

  MyVolume=NewMultiStruct("ART_vol",3,Samples,_RealArray);
  
  MyVolume->Min[0]=MyXYZ->Xmin;
  MyVolume->Min[1]=MyXYZ->Ymin;
  MyVolume->Min[2]=MyXYZ->Zmin;
  MyVolume->Delta[0]=MyXYZ->DeltaX;
  MyVolume->Delta[1]=MyXYZ->DeltaY;
  MyVolume->Delta[2]=MyXYZ->DeltaZ;
  
  VolSignal=MyVolume->Signal;
  RadonSignal=MyRadon->Signal;

  for (iteration=0;iteration<TotalIterations;iteration++) /* Iteration loop */
  {
    if (iteration%writetext==0)
    {
      decay=decay*beta;
      Print(_DNormal,
            "%9i (%9i) done %5.2f %% - decay %5.3f (Iterative_ART_3D)\r",
            iteration,TotalIterations,iteration*procit,decay);
    }

    /* Note this */    
    if ((iteration%(writetext)==0) && (DebugLevel==_DDebug))
    {
/*      WindowVolume(MyVolume,10);
      BoxFilterVolume(MyVolume,2); */
      Print(_DNormal,
            "Iteration %9i. L2-Difference between OrgFile and Estimate %f\n",
            iteration,L2_measure(OrgVolume,MyVolume));

    }

    thetaint=(int)(ThetaSamples*(float)rand()/(RAND_MAX+1.0));

    /* Approximate a weighing scheme for phi */
    phiint=(int)(PhiSamples/4*(float)rand()/(RAND_MAX+1.0))
      +(int)(PhiSamples/4*(float)rand()/(RAND_MAX+1.0))
        +(int)(PhiSamples/4*(float)rand()/(RAND_MAX+1.0))
          +(int)(PhiSamples/4*(float)rand()/(RAND_MAX+1.0));

    uuint=(int)(USamples*(float)rand()/(RAND_MAX+1.0));
    vvint=(int)(VSamples*(float)rand()/(RAND_MAX+1.0));
    radonindex=thetaint+ThetaSamples*(phiint+PhiSamples*(uuint+USamples*vvint));

    if (Interpolation==BL_Interpolation)
      counter=NumericalRadon3D_OneSample_BL(MyWeight,MyTrig,MyXYZ,
                                            thetaint,phiint,uuint,vvint);
    else
      counter=NumericalRadon3D_OneSample_NN(MyWeight,MyTrig,MyXYZ,
                                            thetaint,phiint,uuint,vvint);

    if (counter>2)
    {
      AAsum=Axsum=0.0;

      /* Finding the weight factor */
      for (ii=0;ii<counter;ii++)
      {
        MyWeight2=&MyWeight[ii];
        A=MyWeight2->A;
        AAsum+=A*A;
        Axsum+=A*VolSignal[MyWeight2->VolIndex];
      }
      fract=decay*(RadonSignal[radonindex]-Axsum)/AAsum;

      /* Updating */
      for (ii=0;ii<counter;ii++)
      {
        MyWeight2=&MyWeight[ii];
        VolSignal[MyWeight2->VolIndex]+=fract*(MyWeight2->A);
      }
    }
    /*if (iteration%10000==0) 
      WindowRadial(MyVolume,1);*/
  }
  Print(_DNormal,"All iterations done (Iterative_ART_3D)                                    \n");
  Print(_DNormal,
        "Final L2-Difference between OrgFile and Estimate %f\n",
        L2_measure(OrgVolume,MyVolume));

  return MyVolume;
}


/***************************************************************************
[NAME]
Iterative\_EM\_3D

[SYNOPSIS]
 MultiStruct * Iterative_EM_3D(MultiStruct *,XYZDef *,float)

[DESCRIPTION]
This function will estimate the reconstructed volume using EM
in a three dimensional version. The function will return the reconstructed
volume. The input parameters are: The 4D sinogram, a XYZDef defining the 
sampling of the volume and the last parameter determines the number of 
iterations relative to the number of samples in the 4D sinogram. 

[USAGE]
{\tt MultiStruct * Iterative\_ART\_3D(MultiStruct *MyRadon,XYZDef *MyXYZ,
float maxiterations)}

[REVISION]
23/8-95 Peter Toft
***************************************************************************/
MultiStruct * Iterative_EM_3D(MultiStruct *MyRadon,
                              XYZDef *MyXYZ,float maxiterations)
{
  TrigDef *MyTrig;
  int TotalIterations,iteration;
  int ii;
  int NoOfVolumeSamples,NoOfRadonSamples;
  float procit,tempfl;
  int Samples[3];
  MultiStruct *MyVolume,*TempRadon,*TempVolume,*NormVolume;
  MultiStruct *OrgVolume;
  float *MyVolumeSignal,*MyRadonSignal,*TempRadonSignal,*NormVolumeSignal;
  float *TempVolumeSignal;

  Print(_DNormal,"Initializing (Iterative_EM_3D)\n");
  
  MyTrig=GenerateTrigDef_fromMultiStruct(MyRadon);
  OrgVolume=ReadMult(IniFile3D.OrgFile,IniFile3D.OrgFileType,_Signal);

  Samples[0]=MyXYZ->XSamples;
  Samples[1]=MyXYZ->YSamples;
  Samples[2]=MyXYZ->ZSamples;

  TotalIterations=(int)(maxiterations);

  procit=1e2/TotalIterations;

  MyVolume=NewMultiStruct("EM_Vol",3,Samples,_RealArray);
  MyVolume->Min[0]=MyXYZ->Xmin;
  MyVolume->Min[1]=MyXYZ->Ymin;
  MyVolume->Min[2]=MyXYZ->Zmin;
  MyVolume->Delta[0]=MyXYZ->DeltaX;
  MyVolume->Delta[1]=MyXYZ->DeltaY;
  MyVolume->Delta[2]=MyXYZ->DeltaZ;
 
  NoOfRadonSamples=MyRadon->NoOfSamples;
  NoOfVolumeSamples=MyVolume->NoOfSamples;
  
  TempVolume=DuplicateMultiStruct(MyVolume);
  RenameMultiStruct(TempVolume,"EM_TempVolume");

  TempRadon=DuplicateMultiStruct(MyRadon);
  RenameMultiStruct(TempRadon,"EM_TempRadon");

  MyVolumeSignal=MyVolume->Signal;
  MyRadonSignal=MyRadon->Signal;
  TempRadonSignal=TempRadon->Signal;
  TempVolumeSignal=TempVolume->Signal;

  for (ii=0;ii<NoOfVolumeSamples;ii++) 
    MyVolumeSignal[ii]=1.0;

  for (ii=0;ii<NoOfRadonSamples;ii++) 
    TempRadonSignal[ii]=1.0;

  Print(_DNormal,"Calculating Normalization Volume (Iterative_EM_3D)\n");

  NormVolume=BackProject3D(TempRadon,MyXYZ,MyTrig);
  NormVolumeSignal=NormVolume->Signal;

  for (ii=0;ii<NoOfVolumeSamples;ii++)
    if (fabs(NormVolumeSignal[ii])<1e-7)
      NormVolumeSignal[ii]=1e-7;

  Print(_DNormal,"Iterations starting (Iterative_EM_3D)\n");

  for (iteration=0;iteration<TotalIterations;iteration++) /* Iteration loop */
  {
    Print(_DNormal,
          "Iteration number %i of %i (%5.2f %% done) (Iterative_EM_3D)\n",
          iteration,TotalIterations,iteration*procit);

    NumericalRadon3D_Restore(TempRadon,MyVolume,MyTrig);

    for (ii=0;ii<NoOfRadonSamples;ii++)
    {
      if (fabs(tempfl=TempRadonSignal[ii])>1e-7)
        TempRadonSignal[ii]=MyRadonSignal[ii]/tempfl;
      else
        TempRadonSignal[ii]=MyRadonSignal[ii]; 
    }
    BackProject3D_Restore(TempVolume,TempRadon,MyXYZ,MyTrig);

    for (ii=0;ii<NoOfVolumeSamples;ii++)
      MyVolumeSignal[ii]*=TempVolumeSignal[ii]/NormVolumeSignal[ii];

    Print(_DNormal,
          "Iteration %i. L2-Difference between OrgFile and Estimate %f\n",
          iteration,L2_measure(OrgVolume,MyVolume));

    /*WindowRadial(MyVolume,IniFile3D.Iterative_Par_1);*/

    /*sprintf(Name,"ReconEM_%i",iteration);
    RenameMultiStruct(MyVolume,Name);
    WriteMultiStruct(MyVolume);*/

  }
  Print(_DNormal,"   All iterations done (Iterative_EM_3D)\n");

  return MyVolume;
}


/***************************************************************************
[NAME]
Iterative\_MART\_3D

[SYNOPSIS]
 MultiStruct * Iterative_MART_3D(MultiStruct *,XYZDef *,float)

[DESCRIPTION]
This function will estimate the reconstructed volume using MART
in a three dimensional version. The function will return the reconstructed
volume. The input parameters are: The 4D sinogram, a XYZDef defining the 
sampling of the volume and the last parameter determines the number of 
iterations relative to the number of samples in the 4D sinogram. 

[USAGE]
{\tt MultiStruct * Iterative\_MART\_3D(MultiStruct *MyRadon,XYZDef *MyXYZ,
float maxiterations)}

[REVISION]
19/9-95 Peter Toft
***************************************************************************/
MultiStruct * Iterative_MART_3D(MultiStruct *MyRadon,
                               XYZDef *MyXYZ,float maxiterations)
{
  TrigDef *MyTrig;
  int TotalIterations,iteration,Interpolation;
  int thetaint,phiint,uuint,vvint,counter,statprints;
  int ThetaSamples,PhiSamples,USamples,VSamples,ii,radonindex,writetext;
  int XSamples,YSamples,ZSamples;
  float A,Axsum,fract,procit,rece;
  double decay,beta,alpha;
  MultiStruct *MyVolume,*OrgVolume,*Forward;
  VolWeight *MyWeight,*MyWeight2;
  int Samples[3];
  float *VolSignal,*RadonSignal,*ForwardSignal;

  Print(_DNormal,"Initializing (Iterative_MART_3D)\n");

  MyTrig=GenerateTrigDef_fromMultiStruct(MyRadon);
  Forward=DuplicateMultiStruct(MyRadon);
  srand(37);

  OrgVolume=ReadMult(IniFile3D.OrgFile,IniFile3D.OrgFileType,_Signal);

  ThetaSamples=MyRadon->Samples[0];
  PhiSamples=MyRadon->Samples[1];
  USamples=MyRadon->Samples[2];
  VSamples=MyRadon->Samples[3];

  XSamples=Samples[0]=MyXYZ->XSamples;
  YSamples=Samples[1]=MyXYZ->YSamples;
  ZSamples=Samples[2]=MyXYZ->ZSamples;

  if (maxiterations<0)
  {
    maxiterations=-maxiterations;
    Interpolation=NN_Interpolation;
  }
  else
    Interpolation=BL_Interpolation;
  
  TotalIterations=(int)(maxiterations*ThetaSamples*
                        PhiSamples*USamples*VSamples);

  Print(_DNormal,"TotalIterations = %i\n",TotalIterations);
  statprints=200;
  writetext=TotalIterations/statprints+1;
  procit=1e2/TotalIterations;

  if (fabs(IniFile3D.Iterative_Par_1-IniFile3D.Iterative_Par_2)<1e-3)
  {
    Print(_DNormal,"Illegal iterative decay parameters (Iterative_MART_3D)\n");
    alpha=beta=1;
  }
  else
  {
    alpha=IniFile3D.Iterative_Par_1;
    beta=
      exp(log(IniFile3D.Iterative_Par_2/IniFile3D.Iterative_Par_1)/statprints);
  }
  decay=alpha;

  MyWeight=(VolWeight *)malloc(Interpolation*sizeof(VolWeight)*
                               max(max(XSamples,YSamples),ZSamples));

  MyVolume=NewMultiStruct("MART_vol",3,Samples,_RealArray);
  
  MyVolume->Min[0]=MyXYZ->Xmin;
  MyVolume->Min[1]=MyXYZ->Ymin;
  MyVolume->Min[2]=MyXYZ->Zmin;
  MyVolume->Delta[0]=MyXYZ->DeltaX;
  MyVolume->Delta[1]=MyXYZ->DeltaY;
  MyVolume->Delta[2]=MyXYZ->DeltaZ;
  
  VolSignal=MyVolume->Signal;
  RadonSignal=MyRadon->Signal;
  ForwardSignal=Forward->Signal;

  rece=1.0/exp(1);
  for (ii=0;ii<MyVolume->NoOfSamples;ii++)
    VolSignal[ii]=rece;

  for (iteration=0;iteration<TotalIterations;iteration++) /* Iteration loop */
  {
    if (iteration%writetext==0)
    {
      decay=decay*beta;
      Print(_DNormal,
            "%9i (%9i) done %5.2f %% - decay %5.3f (Iterative_MART_3D)\r",
            iteration,TotalIterations,iteration*procit,decay);
    }

    /* Note this */    
    if (iteration%(writetext*5)==0)
    {
      WindowVolume(MyVolume,10);
      BoxFilterVolume(MyVolume,2); 
      Print(_DNormal,
            "Iteration %9i. L2-Difference between OrgFile and Estimate %f\n",
            iteration,L2_measure(OrgVolume,MyVolume));

    }

    thetaint=(int)(ThetaSamples*(float)rand()/(RAND_MAX+1.0));
    phiint=(int)(PhiSamples/4*(float)rand()/(RAND_MAX+1.0))
      +(int)(PhiSamples/4*(float)rand()/(RAND_MAX+1.0))
        +(int)(PhiSamples/4*(float)rand()/(RAND_MAX+1.0))
          +(int)(PhiSamples/4*(float)rand()/(RAND_MAX+1.0))
;
    uuint=(int)(USamples*(float)rand()/(RAND_MAX+1.0));
    vvint=(int)(VSamples*(float)rand()/(RAND_MAX+1.0));
    /*radonindex=iteration%(ThetaSamples*PhiSamples*USamples*VSamples);*/
    radonindex=thetaint+ThetaSamples*(phiint+PhiSamples*(uuint+USamples*vvint));
    if (Interpolation==BL_Interpolation)
      counter=NumericalRadon3D_OneSample_BL(MyWeight,MyTrig,MyXYZ,
                                            thetaint,phiint,uuint,vvint);
    else
      counter=NumericalRadon3D_OneSample_NN(MyWeight,MyTrig,MyXYZ,
                                            thetaint,phiint,uuint,vvint);
    Axsum=0;
    /* Finding the weight factor */
    for (ii=0;ii<counter;ii++)
    {
      MyWeight2=&MyWeight[ii];
      A=MyWeight2->A;
      Axsum+=A*VolSignal[MyWeight2->VolIndex];
    }
    
    /* Updating */
    for (ii=0;ii<counter;ii++)
    {
      MyWeight2=&MyWeight[ii];
      A=MyWeight2->A;
      if (A>1e-14)
      {
        fract=exp(decay*A*log(A/Axsum));      
        VolSignal[MyWeight2->VolIndex]*=fract;
      }
    }
  }
  Print(_DNormal,"   All iterations done (Iterative_MART_3D)\n");
  return MyVolume;
}


