/******************************************************************************
[HEADER] 

This library contains different common utilities used by most of the
other functions.

******************************************************************************/
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include <stdarg.h>

#include "3dtools.h"
#include "Multi_IO.h"
#include "3dmisc.h"

INI3D IniFile3D;
extern FILE *LogFile;


/*****************************************************************************
[NAME]
GetArg

[SYNOPSIS]
 int GetArg(char *IniBuffer,
            char *Entry,
            char *Value);

[DESCRIPTION]

The function searches the text buffer {\tt IniBuffer} for the string 
{\tt 'Entry='}.  If the entry exists, the text on the right side of the
equal sign will be returned in {\tt Value} as a string.  If the
argument does not exist, the function will return with {\tt
\_ArgError}.

[USAGE]
{\tt Error=GetArg(IniBuffer, "Function", Function);}

Searches for an entry `Function' in {\tt IniBuffer}, and returns its value
in {\tt Function}.

[REVISION]
Sep. 94, JJJ and PAP
*****************************************************************************/ 
int GetArg(char *IniBuffer, char* Entry, char* Value)
{
  char *chrprt, *EntryPrt, *EntryEndPrt, SEntry[101], EEntry[101];
  int ArgError=_ArgError;

  SEntry[0]='\n';
  strcpy(&SEntry[1],Entry);
  strcpy(EEntry,SEntry);
  SEntry[(chrprt=strchr(SEntry,'\0'))-SEntry]=' ';
  chrprt[1]='\0';
  EEntry[(chrprt=strchr(EEntry,'\0'))-EEntry]='=';
  chrprt[1]='\0';
  if ( (chrprt=strstr(IniBuffer,SEntry)) || (chrprt=strstr(IniBuffer,EEntry)) ) {
    chrprt+=1; 
    if (!(EntryPrt=strchr(chrprt,'=')))      
      Error("Error in IniFile: '%s'", Entry);
    if (!(EntryEndPrt=strchr(chrprt,(char) 0x0A)))
      Error("Error in INI file: '%s'", Entry);
    strncpy(Value,(EntryPrt+=1),(EntryEndPrt-EntryPrt));
    Value[EntryEndPrt-EntryPrt]='\0';
    ArgError=_NoError; 
  }
  return(ArgError);
}


/*****************************************************************************
[NAME]
ReadIni

[SYNOPSIS]
 char *ReadIni(char *FileName);

[DESCRIPTION]

This function finds and reads the file {\tt FileName.ini}, and copies
its contents into the return string. 

[USAGE]

{\tt  IniBuffer=ReadIni("Iradon.ini");}

Reads the {\tt .ini} file {\tt Iradon.ini} into {\tt IniBuffer}.

[REVISION]
Sep. 94, JJJ and PAP
*****************************************************************************/
char *ReadIni(char *FileName)
{
  char Temp[100], *chrprt, *IniBuf;
  FILE *InFile;
  int FileSize;

  strcpy(Temp,FileName);
  if(!(chrprt=strrchr(Temp,'.')))
    strcat(Temp,".ini");    

  if (!(InFile=fopen(Temp, "rb"))) 
    Error("Error opening INI file");

  fseek(InFile,0L,SEEK_END);
  FileSize=ftell(InFile);
  rewind(InFile);

  if (!(IniBuf=(char *)malloc(sizeof(char)*FileSize+3)))
    Error("Error allocating memory (ReadIni)");
  IniBuf[0]='\n';
  IniBuf[FileSize+1]='\n';
  IniBuf[FileSize+2]='\0'; 
  if (fread(&IniBuf[1], sizeof(char), FileSize, InFile)!=FileSize)
    Error("Error reading INI file"); 
  return IniBuf;
}




/*****************************************************************************
[NAME]
GetDateTime

[SYNOPSIS]
 void GetDateTime(char *str,
                  int DateTimeFormat);
 
[DESCRIPTION]

This function gets the current time and date, and returns it in the
pointer {\tt str}. The return string are formatted accordingly to {\tt
DateTime} Format. Three formats are avaliable as of now, {\tt
\_LongDate}, {\tt \_Time} and {\tt RealTime}. {\tt \_Longdate} returns the 
date and time in a string as `Day, dd. month, hh:mm:ss', {\tt \_Time} will
return `hh:mm:ss' and {\tt  \_RealTime} returns the number of seconds
from last midnight.

[USAGE]
{\tt GetDateTime(str,\_Time);}

Returns the current time in {\tt str} formatted as ``12:34:56''.

[REVISION]
Oct. 94, JJJ
*****************************************************************************/
void GetDateTime(char *str, int DateTimeFormat)
{
  time_t *timep;
  struct tm *times;

  if (!(timep=(time_t *)malloc(sizeof(time_t))))
    Error(" Memory allocation error (GetDateTime)");
  if (!(times=(struct tm *)malloc(sizeof(struct tm))))
    Error(" Memory allocation error (GetDateTime)");
  time(timep);
  times=localtime(timep);
  if (DateTimeFormat==_LongDate) 
    strftime(str, 55, "%A, %d. %B, %H.%M:%S", times);
  if (DateTimeFormat==_Date) 
    strftime(str, 8, "%y%m%d", times);
  if (DateTimeFormat==_Time) 
    strftime(str, 20,"%H.%M:%S", times);
  if (DateTimeFormat==_RealTime)
    sprintf(str,"%d",times->tm_sec+times->tm_min*60+times->tm_hour*3600);
}


/*****************************************************************************
[NAME]
OpenLog

[SYNOPSIS]
 void OpenLog(char* FileName);

[DESCRIPTION]

This function creates and opens a log file named {\tt
FileName.log}. Appends time and date to the top if the file.  If the
global variable {\tt DebugLevel} equals {\tt \_DHardCore} or {\tt
\_DNolog} no log will be opened. The file handle is the global
variable {\tt LogFile}.

[USAGE]
{\tt OpenLog("Test.log");}

Creates and opens the log file {\tt Test.log}.

[REVISION]
Oct. 94, JJJ
*****************************************************************************/
void OpenLog(char *FileName)
{
  char str[100];
  if (!(DebugLevel&(_DHardCore|_DNoLog))) {  
    strcpy(LogFileName,FileName);
    strcat(LogFileName,".log");
    GetDateTime(str, _LongDate);
    if (!(LogFile=fopen(LogFileName, "w"))) 
      Error("Error opening LOG file");
    fprintf(LogFile,"--- START OF RECON3D LOG; %s ---\n", str);   
    fclose(LogFile);
  }
}


/*****************************************************************************
[NAME]
CloseLog

[SYNOPSIS]
 void CloseLog()

[DESCRIPTION]

This function closes the log file and appends time and date to the end
of the file.

[USAGE]
{\tt CloseLog():}

Closes the current log file.

[REVISION]
Oct. 94, JJJ
*****************************************************************************/
void CloseLog()
{
  char str[100];

  if (!(DebugLevel&(_DHardCore|_DNoLog))) {  
    if (!(LogFile=fopen(LogFileName, "a+"))) 
      Error("Error opening LOG file");
    GetDateTime(str, _Time);
    fprintf(LogFile,"---- Normal termination; %s ---\n", str);   
    fflush(LogFile);
    fclose(LogFile);
  }
}


/*****************************************************************************
[NAME]
Print

[SYNOPSIS]
 void Print(int Level,
            char *fmt,
            ...);

[DESCRIPTION]

This function handles all the writing to the screen and to the
log. {\tt fmt} and {\tt ...} are the same arguments as to {\tt
print}. The variable {\tt \_Level} combined with the global variable
{\tt DebugLevel} chooses what should be written. {\tt DebugLevel}
takes precedence over {\tt Level}. The following choises exist

\begin{tabbing}
{\tt \_DHardCore} \== Nothing written.\\
{\tt \_DSilent}   \>= Nothing written on screen, full log.\\
{\tt \_DNolog}    \>= Full information on screen, no log.\\
{\tt \_DNormal}   \>= Some information on screen, full log.\\
{\tt \_DDebug}    \>= Full information both on screen and in log.
\end{tabbing}

[USAGE]
\verb+Print(_DNormal,"File not found '%%s' \n", TestFileName);+ 

Prints the above message both on the screen and in the log file.

[REVISION]
Oct. 94, JJJ
*****************************************************************************/
void Print(int Level, char *fmt, ...)
{ 
  char LogString[255];
  char str[20];
  va_list ap;

  va_start(ap,fmt);
  vsprintf(LogString, fmt, ap);  
  GetDateTime(str, _Time);
  if ((Level!=_DNoLog) && (!(DebugLevel&(_DHardCore|_DNoLog)))) {
    if (!(LogFile=fopen(LogFileName, "a+"))) 
      Error("Error opening LOG file");
    fprintf(LogFile,"%s: %s",str,LogString);
    fclose(LogFile);
  }
  if (((DebugLevel&(_DDebug|_DNoLog)) && (Level&(_DDebug|_DNoLog|_DNormal))) 
     || ((DebugLevel&(_DNormal)) && (Level&(_DNormal|_DNoLog)))) {
    vprintf(fmt,ap);
    fflush(stdout);
    fflush(LogFile);
  }
  va_end(ap);
} 


/*****************************************************************************
[NAME]
Error

[SYNOPSIS]
 void Error(char *fmt,
            ...);

[DESCRIPTION]

This function is called when a fatal error occurs. It prints the error
message on the screen, closes the log and exits. {\tt fmt} and {\tt
...} are the same arguments as to {\tt print}.

[USAGE]
{\tt Error("Memory allocation problems");}

Prints the above message, closes the log and exits.


[REVISION]
Oct. 94, JJJ and PAP
*****************************************************************************/
void Error(char *fmt, ...)
{
  char LogString[255];
  char str[100];
  va_list ap;

  va_start(ap,fmt);
  vsprintf(LogString, fmt, ap); 
  va_end(ap);

  if (!(DebugLevel&(_DHardCore|_DNoLog))) {
    GetDateTime(str, _Time);
    if (!(LogFile=fopen(LogFileName, "a+"))) 
      printf("Error opening LOG file");
    fprintf(LogFile,"%s: %s\n",str,LogString);
    fprintf(LogFile,"---- IRADON terminated with an error; %s ---\n", str);   
    fclose(LogFile);
  }

  printf(LogString);
  printf("\n");
  exit(1);
}


/**********************************************************
[NAME]
MultReStore

[SYNOPSIS]
 void MultReStore(float *p1,float *p2)

[DESCRIPTION]
The function will multiply the two complex numbers $A$
and $B$ and store the result at the location of $A$. Here
$A=p1[0]~+~i~p1[1]$ and $B=p2[0]~+~i~p2[1]$.

[USAGE]

{\t tMultReStore(arr1,arr2);}

Preforms the multiplication {\tt arr1=arr1*arr2}.

[REVISION]
Nov. 94, JJJ and PT
**********************************************************/
void MultReStore(float *p1,float *p2)
{
  multtemp=p1[0]*p2[0]-p1[1]*p2[1];
  p1[1]=p1[0]*p2[1]+p1[1]*p2[0];
  p1[0]=multtemp;
}


/**********************************************************
[NAME]
MultNew

[SYNOPSIS]
 void MultNew(float *p1,float *p2,float *p3)

[DESCRIPTION] The function will multiply the two complex numbers $A$
and $B$, so that $C=AB$. The result is stored at the location of
$C$. Here $A=p1[0]~+~i~p1[1]$, $B=p2[0]~+~i~p2[1]$ and
$C=p3[0]~+~i~p3[1]$.

[USAGE]
{\tt MultNew(arr1,arr2,arr3);}

Preforms the multiplication {\tt arr3=arr1*arr2}.

[REVISION]
Nov. 94, JJJ and PT 
**********************************************************/
void MultNew(float *p1,float *p2,float *p3)
{
  p3[0]=p1[0]*p2[0]-p1[1]*p2[1];
  p3[1]=p1[0]*p2[1]+p1[1]*p2[0];
}


/*****************************************************************************
[NAME]
FloatVector

[SYNOPSIS]
 float *FloatVector(int Size);

[DESCRIPTION]

Allocates and returns a vector of floats, with {\tt Size} number of
elements. The vector is initialized to 0. Checking is made to ensure
no memory problems.

[USEAGE]
{\tt Test=FloatVector(99);}

Allocates {\tt Test} as a float vector 99 elements long.

[REVISION]
Oct. 94, JJJ
****************************************************************************/
float *FloatVector(int Size)
{
  float *data;

  if (!(data=(float*)calloc(Size,sizeof(float))))
    Error("Memory allocation problems (FloatVector). %i elements.",Size);
  return data;
}


/*****************************************************************************
[NAME]
IntVector

[SYNOPSIS]
 float *IntVector(int Size);

[DESCRIPTION]

Allocates and returns a vector of integers, with {\tt Size} number of
elements. The vector is initialized to 0. Adequate checking is done.

[USEAGE]
{\tt Test=IntVector(99);}

Allocates {\tt Test} as an int vector 99 elements long.

[REVISION]
Oct. 94, JJJ
****************************************************************************/
int *IntVector(int Size)
{
  int *data;

  if (!(data=(int*)calloc(Size,sizeof(int))))
    Error("Memory allocation problems (IntVector). %i elements",Size);
  return data;
}


/*****************************************************************************
[NAME]
convert2

[SYNOPSIS]
void convert2(char *ind);

[DESCRIPTION]

The function swaps the two bytes at the location {\tt ind}. Used for
converting short int's between HP/PC.

[USEAGE]
{\tt convert2((char *)\&X);}

Converts {\tt X}, where {\tt X} is a short int.

[REVISION]
Oct. 94, PAP and PT
*****************************************************************************/
void convert2(char *ind)
{
  char tmp;
 
  tmp=ind[0];
  ind[0]=ind[1];
  ind[1]=tmp;
}


/******************************************************************************
[NAME]
convert4

[SYNOPSIS]
 void convert4(char *ind);

[DESCRIPTION]

The function swaps the four bytes at the location {\tt ind} around the
center. Used for converting short floats between HP/PC.

[USEAGE]
{\tt convert4((char *)\&X);}

Converts {\tt X}, where {\tt X} is a float.

[REVISION]
Oct. 94, PAP and PT
*****************************************************************************/
void convert4(char *ind)
{
  char tmp;

  tmp=ind[0];
  ind[0]=ind[3];
  ind[3]=tmp;
  tmp=ind[1];
  ind[1]=ind[2];
  ind[2]=tmp;
}


/*****************************************************************************
[NAME]
convert8

[SYNOPSIS]
 void convert8(char *ind);

[DESCRIPTION]

The function swaps the eight bytes at the location {\tt ind} around
the center. Used for converting doubles between HP/PC.

[USEAGE]
{\tt convert8((char *)\&X);}

Converts {\tt X}, where {\tt X} is a double.

[REVISION]
Oct. 94, PAP and PT
*****************************************************************************/
void convert8(char *ind)
{
  char tmp;

  tmp=ind[0];
  ind[0]=ind[7];
  ind[7]=tmp;
  tmp=ind[1];
  ind[1]=ind[6];
  ind[6]=tmp;
  tmp=ind[2];
  ind[3]=ind[5];
  ind[5]=tmp;
  tmp=ind[3];
  ind[3]=ind[4];
  ind[4]=tmp;
}


/*****************************************************************************
[NAME]
convert\_UNIX\_PC

[SYNOPSIS]
 void convert_UNIX_PC(char *ind, int lgd);

[DESCRIPTION]

The function swaps the {\tt lgd} bytes at the location {\tt ind}
around the center. Used for converting arbitrary length numbers between
HP/PC.

[USEAGE]
{\tt convert\_UNIX\_PC((char *)\&X, 8);}

Converts {\tt X}, where {\tt X} is 8 bytes long eg. a double.

[REVISION]
Oct. 94, PAP and PT
*****************************************************************************/
void convert_UNIX_PC(char *ind, int lgd)
{
  int i,lgdh,lgdm1;
  char tmp;

  if (lgd>1)
  {
    lgdh=lgd/2;
    lgdm1=lgd-1;
    for (i=0;i<lgdh;i++)
    {
      tmp=ind[i];
      ind[i]=ind[lgdm1-i];
      ind[lgdm1-i]=tmp;
    }
  }
}


/*****************************************************************************
[NAME]
Read3DArgs

[SYNOPSIS]
 void Read3DArgs(void)

[DESCRIPTION]

The function decodes all the information in the inifile and sets 
relevant variables. Uses {\tt IniBuffer}.

[USEAGE]
{\tt Read3DArgs;}


[REVISION]
20/6 1995 Peter And Peter.\\
28/6 1995 Peter Toft (included Iterations parameter).\\
May 11 97 Peter Toft Added reading Select\_Par\_1.
*****************************************************************************/
void Read3DArgs(char *IniBuffer)
{
  char Temp[100], *chrprt;
  int TmpDebugLevel;
  MultiStruct * OrgMulti;
 
  if (!(GetArg(IniBuffer,"DebugLevel",IniFile3D.DebugLevel))) DebugLevel=_DNormal;
  else if (strstr(IniFile3D.DebugLevel,"Normal"))   DebugLevel=_DNormal;
  else if (strstr(IniFile3D.DebugLevel,"Debug"))    DebugLevel=_DDebug;
  else if (strstr(IniFile3D.DebugLevel,"Silent"))   DebugLevel=_DSilent;
  else if (strstr(IniFile3D.DebugLevel,"NoLog"))    DebugLevel=_DNoLog;
  else if (strstr(IniFile3D.DebugLevel,"HardCore")) DebugLevel=_DHardCore;
  else Error("Unknown DebugLevel: '%s'",IniFile3D.DebugLevel);   

  if (GetArg(IniBuffer,"WorkDir",IniFile3D.WorkDir)){
    if(!(IniFile3D.WorkDir[strlen(IniFile3D.WorkDir)-1]=='/'))
      strcat(IniFile3D.WorkDir,"/");}
  else strcpy(IniFile3D.WorkDir,"");
 
  if (!(GetArg(IniBuffer,"Volume",IniFile3D.Volume)))
    Error("'Volume' entry missing in INI file");
  strcpy(Temp,IniFile3D.WorkDir);
  strcat(Temp,IniFile3D.Volume);
  strcpy(IniFile3D.Volume,Temp);
  if((chrprt=strrchr(IniFile3D.Volume,'.'))) {
    strcpy(Temp, chrprt+1);
    *chrprt='\0';}
  if (strstr(Temp,"f3f")) IniFile3D.VolumeType=_F3F;
  else if (strstr(Temp,"a3f")) IniFile3D.VolumeType=_A3F;
  else Error("Unknown file type: '%s'",Temp);
    
 
  OpenLog(IniFile3D.Volume);

  if (!(GetArg(IniBuffer,"Sinogram",IniFile3D.Sinogram)))
    Error("'Sinogram' entry missing in INI file");
  strcpy(Temp,IniFile3D.WorkDir);
  strcat(Temp,IniFile3D.Sinogram);
  strcpy(IniFile3D.Sinogram,Temp);
  if((chrprt=strrchr(IniFile3D.Sinogram,'.'))) {
    strcpy(Temp, chrprt+1);
    *chrprt='\0';}
  if (strstr(Temp,"f4f")) IniFile3D.SinogramType=_F4F;
  else if (strstr(Temp,"a4f")) IniFile3D.SinogramType=_A4F;
  else Error("Unknown file type: '%s'",Temp);


  if (!(GetArg(IniBuffer,"Function",IniFile3D.Function)))
    Error("'Function' entry missing in INI file");

  if ((GetArg(IniBuffer,"OrgFile",IniFile3D.OrgFile))) { 
    strcpy(Temp,IniFile3D.WorkDir);
    strcat(Temp,IniFile3D.OrgFile);
    strcpy(IniFile3D.OrgFile,Temp);
    if(( chrprt=strrchr(IniFile3D.OrgFile,'.'))) {
      strcpy(Temp, chrprt+1);
      *chrprt='\0';}
    if (strstr(Temp,"f3f")) 
      IniFile3D.OrgFileType=_F3F;
    else 
      if (strstr(Temp,"a3f")) 
        IniFile3D.OrgFileType=_A3F;
      else
        if (strstr(Temp,"a4f")) 
          IniFile3D.OrgFileType=_A4F;
        else
          if (strstr(Temp,"f4f")) 
            IniFile3D.OrgFileType=_F4F;
          else 
            Error("Unknown file type: '%s'",Temp);

    if ((IniFile3D.OrgFileType==_F3F) || (IniFile3D.OrgFileType==_A3F))
    {
      TmpDebugLevel=DebugLevel;
      DebugLevel=_DHardCore;
      OrgMulti=ReadMult(IniFile3D.OrgFile,IniFile3D.OrgFileType,_NoSignal);
      DebugLevel=TmpDebugLevel;
      
      IniFile3D.Xmin=OrgMulti->Min[0];
      IniFile3D.Ymin=OrgMulti->Min[1];
      IniFile3D.Zmin=OrgMulti->Min[2];
      
      IniFile3D.XSamples=OrgMulti->Samples[0];
      IniFile3D.YSamples=OrgMulti->Samples[1];
      IniFile3D.ZSamples=OrgMulti->Samples[2];
      
      IniFile3D.DeltaX=OrgMulti->Delta[0];
      IniFile3D.DeltaY=OrgMulti->Delta[1];
      IniFile3D.DeltaZ=OrgMulti->Delta[2];
      FreeMultiStruct(OrgMulti);
    }
  }
  else 
  {
    strcpy(IniFile3D.OrgFile,"");
    
    if (!(GetArg(IniBuffer,"Xmin",Temp)))
      Error("'Xmin' entry missing in INI file");
    sscanf(Temp,"%f",&(IniFile3D.Xmin));
    
    if (GetArg(IniBuffer,"Ymin",Temp))
      sscanf(Temp,"%f",&(IniFile3D.Ymin));
    else IniFile3D.Ymin=IniFile3D.Xmin;
    
    if (GetArg(IniBuffer,"Zmin",Temp))
      sscanf(Temp,"%f",&(IniFile3D.Zmin));
    else IniFile3D.Zmin=IniFile3D.Xmin;
    
    if (!(GetArg(IniBuffer,"DeltaX",Temp)))
      Error("'DeltaX' entry missing in INI file");
    sscanf(Temp,"%f",&(IniFile3D.DeltaX));
    
    if (GetArg(IniBuffer,"DeltaY",Temp))
      sscanf(Temp,"%f",&(IniFile3D.DeltaY));
    else IniFile3D.DeltaY=IniFile3D.DeltaX;
    
    if (GetArg(IniBuffer,"DeltaZ",Temp))
      sscanf(Temp,"%f",&(IniFile3D.DeltaZ));
    else IniFile3D.DeltaZ=IniFile3D.DeltaX;
    
    if (!(GetArg(IniBuffer,"XSamples",Temp)))
      Error("'XSamples' entry missing in INI file");
    sscanf(Temp,"%d",&(IniFile3D.XSamples));
    
    if (GetArg(IniBuffer,"YSamples",Temp))
      sscanf(Temp,"%d",&(IniFile3D.YSamples));
    else IniFile3D.YSamples=IniFile3D.XSamples;
    
    if (GetArg(IniBuffer,"ZSamples",Temp))
      sscanf(Temp,"%d",&(IniFile3D.ZSamples));
    else IniFile3D.ZSamples=IniFile3D.XSamples;
  }

  if (!(GetArg(IniBuffer,"Iterations",Temp)))
    IniFile3D.Iterations=0.0;
  else
    sscanf(Temp,"%f",&(IniFile3D.Iterations));

  if (!(GetArg(IniBuffer,"Iterative_Par_1",Temp)))
    IniFile3D.Iterative_Par_1=0.0;
  else
    sscanf(Temp,"%f",&(IniFile3D.Iterative_Par_1));

  if (!(GetArg(IniBuffer,"Iterative_Par_2",Temp)))
    IniFile3D.Iterative_Par_2=0.0;
  else
    sscanf(Temp,"%f",&(IniFile3D.Iterative_Par_2));

  if (!(GetArg(IniBuffer,"Direct_Par_1",Temp)))
    IniFile3D.Direct_Par_1=0.0;
  else
    sscanf(Temp,"%f",&(IniFile3D.Direct_Par_1));

  if (!(GetArg(IniBuffer,"Select_Par_1",Temp)))
    IniFile3D.Select_Par_1=0;
  else
    sscanf(Temp,"%i",&(IniFile3D.Select_Par_1));
}


/****************************************************************
[NAME]
NextRadix2

[SYNOPSIS]
 int NextRadix2(int i)

[DESCRIPTION]
The function will return the lowest integer which is a power
of two higher of equal to {\tt i}.

[USAGE]
{\tt Mypow2=NextRadix2(i)}

[NOTE]
See {\tt NextRadix2Vector} for a vectororized function.

[REVISION]
20/5-95 Peter Toft
****************************************************************/
int NextRadix2(int i)
{
  return (int)floor(0.5+exp(M_LN2*ceil(log(i)/M_LN2)));
}


/****************************************************************
[NAME]
NextRadix2Vector

[SYNOPSIS]
 int * NextRadix2(int *vec,int Dim)

[DESCRIPTION]
The function implements a vectorized version of {\tt NextRadix2}

[USAGE]
{\tt Mypow2=NextRadix2(i)}

[NOTE]
See {\tt NextRadix2} for a scalar function.

[REVISION]
20/5-95 Peter Toft
****************************************************************/
int *NextRadix2Vector(int *vec,int Dim)
{
  int i;
  int *revec;
  revec=IntVector(Dim);
  for (i=0;i<Dim;i++)
    revec[i]=NextRadix2(vec[i]);
  return revec;
}


/****************************************************************
[NAME]
MinFloatVector

[SYNOPSIS]
 float * MinFloatVector(float *vec,int Samples)

[DESCRIPTION]
The function returns the Minimum of the {\tt vec}, with 
number of samples {\tt Samples}.

[USAGE]
{\tt Minvec=MinFloatVector(vec,100)}

Will return the Min element of {\tt vec}, which has 100 elements.

[NOTE]
See {\tt MaxFloatVector} for the minimal value.

[REVISION]
Dec. 14 - 95 Peter Toft
****************************************************************/
float MinFloatVector(float *vec,int Samples)
{
  int i;
  float minimum;

  if (Samples<1)
    Error("The vector whould have elements (MinFloatVector)");

  minimum=vec[0];
  for (i=1;i<Samples;i++)
    if (vec[i]<minimum)
      minimum=vec[i];

  return minimum;
}


/****************************************************************
[NAME]
MaxFloatVector

[SYNOPSIS]
 float * MaxFloatVector(float *vec,int Samples)

[DESCRIPTION]
The function returns the Maximum of the {\tt vec}, with 
number of samples {\tt Samples}.

[USAGE]
{\tt Maxvec=MaxFloatVector(vec,100)}

Will return the Max element of {\tt vec}, which has 100 elements.

[NOTE]
See {\tt MinFloatVector} for the minimal value.

[REVISION]
Dec. 14 - 95 Peter Toft\\
Dec. 15 - 95 Tiny error corrected.
****************************************************************/
float MaxFloatVector(float *vec,int Samples)
{
  int i;
  float maximum;

  if (Samples<1)
    Error("The vector whould have elements (MaxFloatVector)");

  maximum=vec[0];
  for (i=1;i<Samples;i++)
    if (vec[i]>maximum)
      maximum=vec[i];

  return maximum;
}


/****************************************************************
[NAME]
MinRealComplexVector

[SYNOPSIS]
 float * MinRealComplexVector(float *vec,int Samples)

[DESCRIPTION]
The function returns the Minimum of the realpart of {\tt vec}, with 
number of samples {\tt Samples}.

[USAGE]
{\tt Minval=MinRealComplexVector(vec,100)}

Will return the Min element of {\tt vec}, which has 100 complex
elements.

[NOTE]
See {\tt MaxRealComplexVector} for the minimal value.

[REVISION]
Dec. 17 - 95 Peter Toft
****************************************************************/
float MinRealComplexVector(float *vec,int Samples)
{
  int i;
  float minimum;

  if (Samples<1)
    Error("The vector whould have elements (MinFloatVector)");

  minimum=vec[0];
  for (i=1;i<Samples;i++)
    if (vec[2*i]<minimum)
      minimum=vec[2*i];

  return minimum;
}


/****************************************************************
[NAME]
MaxRealComplexVector

[SYNOPSIS]
 float * MaxRealComplexVector(float *vec,int Samples)

[DESCRIPTION]
The function returns the Maximum of the realpart of {\tt vec}, with 
number of samples {\tt Samples}.

[USAGE]
{\tt Maxval=MaxRealComplexVector(vec,100)}

Will return the Max element of {\tt vec}, which has 100 complex
elements.

[NOTE]
See {\tt MaxRealComplexVector} for the maximal value.

[REVISION]
Dec. 17 - 95 Peter Toft
****************************************************************/
float MaxRealComplexVector(float *vec,int Samples)
{
  int i;
  float maximum;

  if (Samples<1)
    Error("The vector whould have elements (MaxFloatVector)");

  maximum=vec[0];
  for (i=1;i<Samples;i++)
    if (vec[2*i]>maximum)
      maximum=vec[2*i];

  return maximum;
}


/****************************************************************
[NAME]
MinImagComplexVector

[SYNOPSIS]
 float * MinImageComplexVector(float *vec,int Samples)

[DESCRIPTION]
The function returns the Minimum of the Imagepart of {\tt vec}, with 
number of samples {\tt Samples}.

[USAGE]
{\tt Minval=MinImageComplexVector(vec,100)}

Will return the Min element of {\tt vec}, which has 100 complex
elements.

[NOTE]
See {\tt MaxImageComplexVector} for the minimal value.

[REVISION]
Dec. 17 - 95 Peter Toft
****************************************************************/
float MinImagComplexVector(float *vec,int Samples)
{
  int i;
  float minimum;

  if (Samples<1)
    Error("The vector whould have elements (MinFloatVector)");

  minimum=vec[1];
  for (i=1;i<Samples;i++)
    if (vec[2*i+1]<minimum)
      minimum=vec[2*i+1];

  return minimum;
}


/****************************************************************
[NAME]
MaxImagComplexVector

[SYNOPSIS]
 float * MaxImagComplexVector(float *vec,int Samples)

[DESCRIPTION]
The function returns the Maximum of the Imagpart of {\tt vec}, with 
number of samples {\tt Samples}.

[USAGE]
{\tt Maxval=MaxImagComplexVector(vec,100)}

Will return the Max element of {\tt vec}, which has 100 complex
elements.

[NOTE]
See {\tt MaxImagComplexVector} for the maximal value.

[REVISION]
Dec. 17 - 95 Peter Toft
****************************************************************/
float MaxImagComplexVector(float *vec,int Samples)
{
  int i;
  float maximum;

  if (Samples<1)
    Error("The vector whould have elements (MaxFloatVector)");

  maximum=vec[1];
  for (i=1;i<Samples;i++)
    if (vec[2*i+1]>maximum)
      maximum=vec[2*i+1];

  return maximum;
}


/****************************************************************
[NAME]
ZeroFloatVector

[SYNOPSIS]
 void ZeroFloatVector(float *vec,int Samples)

[DESCRIPTION]
The function zeros the vector {\tt vec}, with number of 
samples {\tt Samples}.

[USAGE]
{\tt ZeroFloatVector(vec,100)}

Will zero the vector {\tt vec}, which has 100 elements.

[REVISION]
Dec. 19 - 95 Peter Toft
****************************************************************/
void ZeroFloatVector(float *vec,int Samples)
{
  int i;

  if (Samples<1)
    Error("The vector whould have elements (ZeroFloatVector)");

  for (i=0;i<Samples;i++)
    vec[i]=0.0;
}










