/*****************************************************************
[HEADER]
This toolbox contains functions for manipulation of multi 
dimensional structures, i.e., {\tt MultiStruct} defined in 
{\tt 3dtools.h}.
\vspace*{5mm}

{\large The MultiStruct}
\vspace*{5mm}

\begin{tabular}{ll}
  int   Id;              & Id used to restore MultiStructs            \\ 
  int   LengthOfHeader;  & Length of header in bytes before any vector \\
  int   Dim;             & Number of dimensions                        \\
  int   NoOfSamples;     & Total number of samples                     \\
  int   Type;		 & Picture type (eg. format on disc, domain)   \\
  char  FileName[160];	 & Name used for saving/restoring this image	\\
  char  Description[80]; & eg. short description of image (optional)	\\
  char  Date[8];	 & Date (DDMMYY) (optional)			\\
  int   ArrayType;	 & Defines number format: Complex(2) or Real(1) \\
  int   *Samples;        & Pointer to number of samples. Length is Dim \\
  float *Min;            & Pointer to min positions. Length is Dim     \\
  float *Delta;          & Pointer to sampling distances. Length is Dim \\
  float *Signal;         & Pointer to signal. Length: product of dims  
\end{tabular}								  

\vspace*{5mm} 

In the {\tt f<Dim>f-files}, the above header is written first, then
the vectors {\tt *Samples}, {\tt *min}, {\tt *Delta}, each of length
{\tt Dim}. 

Finally a vector {\tt *Signal} with the multi dimensional signal
values. 

For a three dimensional volume indexed by {\tt (xi,xy,zi)}
the signal values are stored as {\tt Signal[xi+X*(yi+Y*zi)]}.

\vspace*{5mm}
Peter Toft and Peter Philipsen

Dep. of Mathematical Modelling, Building 349,

Technical University of Denmark,

2800 Lyngby

Denmark

ptoft,pap@ei.dtu.dk
*****************************************************************/
#include <math.h>
#include <time.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <malloc.h>
#include "3dtools.h"
#include "3dmisc.h"


/****************************************************************
[NAME]
DuplicateMultiStruct

[SYNOPSIS]
MultiStruct* DuplicateMultiStruct(MultiStruct *MyMultiStruct)

[DESCRIPTION]

This function will dublicate a {\tt MultiStruct}.

[USAGE]
{\tt MyNewMultiStruct=DuplicateMultiStruct(MyOldMultiStruct);}

[REVISION]
20/5-95 Peter Toft
****************************************************************/
MultiStruct* DuplicateMultiStruct(MultiStruct *MyMultiStruct)
{
  int m;
  MultiStruct *MyNewMultiStruct;
  Print(_DDebug,"DuplicateMultiStruct\n");

  MyNewMultiStruct=NewMultiStruct(MyMultiStruct->FileName,
				  MyMultiStruct->Dim,
				  MyMultiStruct->Samples,
				  MyMultiStruct->ArrayType);

  for (m=0;m<MyMultiStruct->Dim;m++)
  {
    MyNewMultiStruct->Delta[m]=MyMultiStruct->Delta[m];
    MyNewMultiStruct->Min[m]=MyMultiStruct->Min[m];
  }
  for (m=0;m<MyMultiStruct->NoOfSamples*MyMultiStruct->ArrayType;m++)
    MyNewMultiStruct->Signal[m]=MyMultiStruct->Signal[m];

  return MyNewMultiStruct;
}


/****************************************************************
[NAME]
RenameMultiStruct

[SYNOPSIS]
void RenameMultiStruct(MultiStruct *MyMultiStruct, char *NewFileName)

[DESCRIPTION]

This function will set the {\tt MultiStruct}-filename to 
{\tt NewFileName}.

[USAGE]
{\tt RenameMultiStruct(MyMultiStruct, "Multi");}

[REVISION]
20/5-95 Peter Toft
****************************************************************/
void RenameMultiStruct(MultiStruct *MyMultiStruct, char *NewFileName)
{
  Print(_DDebug,"RenameMultiStruct: Renaming `%s' to `%s'\n",
          MyMultiStruct->FileName, NewFileName);
  strcpy(MyMultiStruct->FileName,NewFileName);
}


/****************************************************************
[NAME]
ZeroMultiStruct

[SYNOPSIS]
void ZeroMultiStruct(MultiStruct * MyMultiStruct)


[DESCRIPTION]

This function will set the signal vector in a {\tt MultiStruct}
to zero.

[USAGE]
{\tt ZeroMultiStruct(MyMultiStruct);}

[REVISION]
20/5-95 Peter Toft
****************************************************************/
void ZeroMultiStruct(MultiStruct * MyMultiStruct)
{
  int m;
  float *sig;

  Print(_DDebug,"ZeroMultiStruct: Zeroing `%s'\n", MyMultiStruct->FileName);

  sig=MyMultiStruct->Signal;
  for (m=0;m<MyMultiStruct->NoOfSamples*MyMultiStruct->ArrayType;m++)
    sig[m]=0.0;
}


/****************************************************************
[NAME]
AbsMultiStruct

[SYNOPSIS]
void AbsMultiStruct(MultiStruct * MyMultiStruct)


[DESCRIPTION]

This function convert the signal in a complex valued {\tt MultiStruct} 
to its real valued absolute.

[USAGE]
{\tt AbsMultiStruct(MyMultiStruct);}

[NOTE]
See also 
{\tt MinMultiStruct, MaxMultiStruct, RealMultiStruct, ImagMultiStruct}


[REVISION]
20/5-95 Peter Toft and Peter Philipsen
****************************************************************/
void AbsMultiStruct(MultiStruct * MyMultiStruct)
{
  int i,m;
  float *NewSignal,*sig,re,im;

  Print(_DDebug,"AbsMultiStruct: `%s'\n", MyMultiStruct->FileName);
  
  if(MyMultiStruct->ArrayType!=_RealArray)
  {
    NewSignal=FloatVector(MyMultiStruct->NoOfSamples*(_RealArray));
 
    sig=MyMultiStruct->Signal;
 
    for (m=0;m<MyMultiStruct->NoOfSamples;m++)
    {
      re=sig[i=(m<<1)];
      im=sig[i+1];
      NewSignal[m]=sqrt(re*re+im*im);
    }
    MyMultiStruct->ArrayType=_RealArray;
    Free(sig);
    MyMultiStruct->Signal=NewSignal;
  }
}


/****************************************************************
[NAME]
ImagMultiStruct

[SYNOPSIS]
void ImagMultiStruct(MultiStruct * MyMultiStruct)


[DESCRIPTION]

This function convert the signal in a complex valued {\tt MultiStruct} 
to its imaginary part.

[USAGE]
{\tt ImagMultiStruct(MyMultiStruct);}

[REVISION]
20/5-95 Peter Toft and Peter Philipsen
****************************************************************/
void ImagMultiStruct(MultiStruct * MyMultiStruct)
{
  int m;
  float *NewSignal,*sig;

  Print(_DDebug,"ImagMultiStruct: `%s'\n", MyMultiStruct->FileName);
  
  if(MyMultiStruct->ArrayType!=_RealArray)
  {
    NewSignal=FloatVector(MyMultiStruct->NoOfSamples*(_RealArray));
 
    sig=MyMultiStruct->Signal;
 
    for (m=0;m<MyMultiStruct->NoOfSamples;m++)
      NewSignal[m]=sig[(m<<1)+1];

    MyMultiStruct->ArrayType=_RealArray;
    Free(sig);
    MyMultiStruct->Signal=NewSignal;
  }  
}


/****************************************************************
[NAME]
RealMultiStruct

[SYNOPSIS]
void RealMultiStruct(MultiStruct * MyMultiStruct)

[DESCRIPTION]

This function convert the signal in a complex valued {\tt MultiStruct} 
to its real part.

[USAGE]
{\tt RealMultiStruct(MyMultiStruct);}

[REVISION]
20/5-95 Peter Toft and Peter Philipsen
****************************************************************/
void RealMultiStruct(MultiStruct * MyMultiStruct)
{
  int m;
  float *NewSignal,*sig;

  Print(_DDebug,"RealMultiStruct: `%s'\n", MyMultiStruct->FileName);
  
  if(MyMultiStruct->ArrayType!=_RealArray)
  {
    NewSignal=FloatVector(MyMultiStruct->NoOfSamples);
 
    sig=MyMultiStruct->Signal;
 
    for (m=0;m<MyMultiStruct->NoOfSamples;m++)
      NewSignal[m]=sig[(m<<1)];

    MyMultiStruct->ArrayType=_RealArray;
    Free(sig);
    MyMultiStruct->Signal=NewSignal;
  }
}


/****************************************************************
[NAME]
Real2ComplexMultiStruct

[SYNOPSIS]
void Real2ComplexMultiStruct(MultiStruct * MyMultiStruct)

[DESCRIPTION]

This function convert the signal in a real valued {\tt MultiStruct} 
to a complex signal with imaginary part equal zero.

[USAGE]
{\tt RealMultiStruct(MyMultiStruct);}

[REVISION]
20/5-95 Peter Toft and Peter Philipsen
****************************************************************/
void Real2ComplexMultiStruct(MultiStruct * MyMultiStruct)
{
  int m,l;
  float *NewSignal,*sig;

  Print(_DDebug,"Real2ComplexMultiStruct: `%s'\n", MyMultiStruct->FileName);
  
  if(MyMultiStruct->ArrayType==_RealArray)
  {
    NewSignal=FloatVector(MyMultiStruct->NoOfSamples*(_ComplexArray));
 
    sig=MyMultiStruct->Signal;
    l=0;
    for (m=0;m<MyMultiStruct->NoOfSamples;m++)
    {
      NewSignal[l++]=sig[m];
      NewSignal[l++]=0.0;
    }
    MyMultiStruct->ArrayType=_ComplexArray;
    Free(MyMultiStruct->Signal);
    MyMultiStruct->Signal=NewSignal;
  }
  
}


/****************************************************************
[NAME]
NewMultiStruct

[SYNOPSIS]
MultiStruct *NewMultiStruct(char *FileName,
			    int Dimensions,
			    int *Samples,
			    int ArrayType)

[DESCRIPTION]

This function returns a {\tt MultiStruct} with a number a dimensions
given in {\tt Dimensions}. The number of samples in each direction
is given in {\tt Samples} and {\tt ArrayType} is {\tt \_ComplexArray}
or {\tt \_RealArray}. 

[USAGE]
{\tt int Samples[3];

Samples[0]=2;
Samples[1]=10;
Samples[2]=5;

MyMultiStruct=NewMultiStruct("My3D",3,Samples,\_RealArray);}

This will return a three dimensional {\tt MultiStruct} with $2\times10\times5$ samples.

[REVISION]
20/5-95 Peter Toft
****************************************************************/
MultiStruct *NewMultiStruct(char *FileName,
			    int Dimensions,
			    int *Samples,
			    int ArrayType)
{
  int m,size;
  MultiStruct *MyMultiStruct;
  char Value[8];

  Print(_DDebug,"NewMultiStruct %d dimensions: Allocating memory for `%s'\n",
	Dimensions,
	FileName);

  if(!(MyMultiStruct=(MultiStruct *) malloc(sizeof(MultiStruct)))) 
    Error( "Memory allocation problems (NewMultiStruct)" );
  MyMultiStruct->Id=_00ND;
  MyMultiStruct->LengthOfHeader=sizeof(MultiStruct);
  MyMultiStruct->Dim=Dimensions;

  GetDateTime(Value,_Date);
  strncpy(MyMultiStruct->Date,Value,6);
  MyMultiStruct->Date[6]='\0';
  size=1;
  MyMultiStruct->Samples=IntVector(Dimensions);
  MyMultiStruct->Min=FloatVector(Dimensions);
  MyMultiStruct->Delta=FloatVector(Dimensions);
  for (m=0;m<Dimensions;m++)
  {
    size*=MyMultiStruct->Samples[m]=Samples[m];
    MyMultiStruct->Min[m]=0.0;
    MyMultiStruct->Delta[m]=1.0;
  }

  MyMultiStruct->NoOfSamples=size; 
  strcpy(MyMultiStruct->FileName,FileName);
  MyMultiStruct->ArrayType=ArrayType;

  MyMultiStruct->Signal=FloatVector(size*ArrayType); 

  return MyMultiStruct;
}


/****************************************************************
[NAME]
NewMultiStruct\_NoSignal

[SYNOPSIS]
MultiStruct *NewMultiStruct_NoSignal(char *FileName,
				     int Dimensions,
				     int *Samples,
				     int ArrayType)

[DESCRIPTION]

This function returns a {\tt MultiStruct} with a number a dimensions
given in {\tt Dimensions}. The number of samples in each direction is
given in {\tt Samples} and {\tt ArrayType} is {\tt \_ComplexArray} or
{\tt \_RealArray}. The {\tt MultiStruct} contains no signal. Use, e.g.,
to read header.

[USAGE]
{\tt int Samples[3];

Samples[0]=2;
Samples[1]=10;
Samples[2]=5;

MyMultiStruct=NewMultiStruct\_NoSignal("My3D",3,Samples,\_RealArray);}

This will return a three dimensional {\tt MultiStruct} with
$2\times10\times5$ samples without signal.

[REVISION]
20/5-95 Peter Toft
****************************************************************/
MultiStruct *NewMultiStruct_NoSignal(char *FileName,
				     int Dimensions,
				     int *Samples,
				     int ArrayType)
{
  int m,size;
  MultiStruct *MyMultiStruct;

  Print(_DDebug,"NewMultiStruct %d dimensions: Allocating memory for `%s'\n",
	Dimensions,
	FileName);

  if(!(MyMultiStruct=(MultiStruct *) malloc(sizeof(MultiStruct)))) 
    Error( "Memory allocation problems (NewMultiStruct)" );
  MyMultiStruct->Id=_00ND;
  MyMultiStruct->LengthOfHeader=sizeof(MultiStruct);
  MyMultiStruct->Dim=Dimensions;
  size=1;
  MyMultiStruct->Samples=IntVector(Dimensions);
  MyMultiStruct->Min=FloatVector(Dimensions);
  MyMultiStruct->Delta=FloatVector(Dimensions);
  for (m=0;m<Dimensions;m++)
  {
    size*=MyMultiStruct->Samples[m]=Samples[m];
    MyMultiStruct->Min[m]=0.0;
    MyMultiStruct->Delta[m]=1.0;
  }

  MyMultiStruct->NoOfSamples=size; 
  strcpy(MyMultiStruct->FileName,FileName);
  MyMultiStruct->ArrayType=ArrayType;
  MyMultiStruct->Signal=NULL;

  return MyMultiStruct;
}


/****************************************************************
[NAME]
FreeMultiStruct

[SYNOPSIS]
void FreeMultiStruct(MultiStruct *MyMultiStruct)

[DESCRIPTION]

The function free the memory allocated by the {\tt MultiStruct}.
The function can also be used to free a {\tt MultiStruct} with
no signal allocated.

[USAGE] 
{\tt FreeMultiStruct(MyMultiStruct);}

[REVISION]
20/5-95 Peter Toft
****************************************************************/
void FreeMultiStruct(MultiStruct *MyMultiStruct)
{
  if(MyMultiStruct->Signal!=NULL)
    Free(MyMultiStruct->Signal);
  Free(MyMultiStruct->Samples);
  Free(MyMultiStruct->Min);
  Free(MyMultiStruct->Delta);
  Free(MyMultiStruct);
}


/****************************************************************
[NAME]
MinMultiStruct

[SYNOPSIS]
float MinMultiStruct(MultiStruct *MyMultiStruct)

[DESCRIPTION]

The function return the minimum signal value in the 
{\tt MultiStruct}. If complex valued the minimum of the
absolute signal is returned.

[USAGE] 
{\tt minsig=MinMultiStruct(MyMultiStruct);}

[NOTE]
See also {\tt MaxMultiStruct, AbsMultiStruct}

[REVISION]
20/5-95 Peter Toft
****************************************************************/
float MinMultiStruct(MultiStruct *MyMultiStruct)
{
  int i;
  float thismax,ff,re,im;

  if (MyMultiStruct->Signal==NULL)
    thismax=0;
  else
  {
    if (MyMultiStruct->ArrayType==_RealArray)
    {
      thismax=MyMultiStruct->Signal[0];
      for (i=0;i<MyMultiStruct->NoOfSamples;i++)
	if ((ff=MyMultiStruct->Signal[i])<thismax) 
	  thismax=ff;
    }
    else
    {
      re=MyMultiStruct->Signal[0];
      im=MyMultiStruct->Signal[1];
      thismax=sqrt(re*re+im*im);
      for (i=0;i<MyMultiStruct->NoOfSamples;i++)
      {
	re=MyMultiStruct->Signal[i+i];
	im=MyMultiStruct->Signal[i+i+1];
	ff=sqrt(re*re+im*im);
	if (ff<thismax) 
	  thismax=ff;
      }
    }
  }
  return thismax;
}


/****************************************************************
[NAME]
MaxMultiStruct

[SYNOPSIS]
float MaxMultiStruct(MultiStruct *MyMultiStruct)

[DESCRIPTION]

The function return the maximum signal value in the 
{\tt MultiStruct}. If complex valued the maximum of the
absolute signal is returned.

[USAGE] 
{\tt maxsig=MaxMultiStruct(MyMultiStruct);}

[NOTE]
See also {\tt MinMultiStruct, AbsMultiStruct}

[REVISION]
20/5-95 Peter Toft
****************************************************************/
float MaxMultiStruct(MultiStruct *MyMultiStruct)
{
  int i;
  float thismax,ff,re,im;

  if (MyMultiStruct->Signal==NULL)
    thismax=0;
  else
  {
    if (MyMultiStruct->ArrayType==_RealArray)
    {
      thismax=MyMultiStruct->Signal[0];
      for (i=0;i<MyMultiStruct->NoOfSamples;i++)
	if ((ff=MyMultiStruct->Signal[i])>thismax) 
	  thismax=ff;
    }
    else
    {
      re=MyMultiStruct->Signal[0];
      im=MyMultiStruct->Signal[1];
      thismax=sqrt(re*re+im*im);
      for (i=0;i<MyMultiStruct->NoOfSamples;i++)
      {
	re=MyMultiStruct->Signal[i+i];
	im=MyMultiStruct->Signal[i+i+1];
	ff=sqrt(re*re+im*im);
	if (ff>thismax) 
	  thismax=ff;
      }
    }
  }
  return thismax;
}


/****************************************************************
[NAME]
InfoMultiStruct

[SYNOPSIS]
void InfoMultiStruct(int DebugLevel,MultiStruct *MyMultiStruct)

[DESCRIPTION]

The function returns informations of the {\tt MyMultiStruct}
such as number of samples, minimum and maximum of the signal. 
{\tt DebugLevel} controls the output level.

[USAGE] 
{\tt MultiStruct(\_DNormal,MyMultiStruct);}

[REVISION]
20/5-95 Peter Toft\\
14/6-95 Peter and Peter (Included debugLevel)
****************************************************************/
void InfoMultiStruct(int MyDebug,MultiStruct *MyMultiStruct)
{
  int i;

  Print(MyDebug,"------- InfoMultiStruct -------\n");
  Print(MyDebug,"Name of MultiStruct : %s\n",MyMultiStruct->FileName);
  Print(MyDebug,"Date : %s\n",MyMultiStruct->Date);
  Print(MyDebug,"Number of Dimensions : %i\n",MyMultiStruct->Dim);
  Print(MyDebug,"Number of Samples : %i\n",MyMultiStruct->NoOfSamples);
  Print(MyDebug,"Length of Header : %i\n",MyMultiStruct->LengthOfHeader);
  if (MyMultiStruct->ArrayType==_RealArray)
  {
    Print(MyDebug,"The array if a RealArray\n");
    if(MyMultiStruct->Signal!=NULL)
    {
      Print(MyDebug,"Minimum of Signal : %e\n",MinMultiStruct(MyMultiStruct));
      Print(MyDebug,"Maximum of Signal : %e\n",MaxMultiStruct(MyMultiStruct));
    }
    else
      Print(MyDebug,"Signal not current.\n");
  }
  else
  {
    Print(MyDebug,"The array if a ComplexArray\n");
    if(MyMultiStruct->Signal!=NULL)
    {
      Print(MyDebug,"Minimum of Absolute Signal : %e\n",
	    MinMultiStruct(MyMultiStruct));
      Print(MyDebug,"Maximum of Absolute Signal : %e\n",
	    MaxMultiStruct(MyMultiStruct));
    }
    else
      Print(MyDebug,"Signal not current.\n");
  }
   
  for (i=0;i<MyMultiStruct->Dim;i++)
    Print(MyDebug,"Dimension %i : Samples = %4i  Min = %+e  Delta = %+e\n",i,
	  MyMultiStruct->Samples[i],
	  MyMultiStruct->Min[i],
	  MyMultiStruct->Delta[i]);
  Print(MyDebug,"-------------------------------\n");
}





/****************************************************************
[NAME]
ResizeMultiStruct

[SYNOPSIS]
MultiStruct * ResizeMultiStruct(MultiStruct *MyMultiStruct, 
				int *NewSize,
				int *NewOrder)

[DESCRIPTION] 
This function will return a resized multistruct. The funtion can both
be used to crop (chop values off) and expand (add zeros) corresponding
to the number of samples given in {\tt NewSize}, which must be of same
length as the number of dimensions in {\tt MyMultiStruct}. The vector
{\tt NewOrder} with same length controls the data-adjustment. If the
i'th index is -1 the data is moved to the lower end, 0 to the middle
and 1 implies that data is right adjusted.
 
[USAGE] 
{\tt int NewSize[3],NewOrder[3];
NewSize[0]=5;
NewSize[1]=15;
NewSize[2]=25;
NewOrder[0]=0;
NewOrder[1]=0;
NewOrder[2]=0;
ResizeMultiStruct(MyMultistruct,MyNewSize,MyOrder);}

This will return a $5 \times 15 \times 25$ {\tt MultiStruct}.

[REVISION]
20/5-95 Peter Philipsen and Peter Toft
****************************************************************/
MultiStruct * ResizeMultiStruct(MultiStruct *MyMultiStruct, 
				int *NewSize,
				int *NewOrder)
{
  MultiStruct *NewStruct;
  int j,*LowerIn,*LowerOut,*Dim;
  int ProductOfDim,tmp,*Xdim,i,Number_New,Number_Old;
   
  LowerIn=IntVector(MyMultiStruct->Dim);
  LowerOut=IntVector(MyMultiStruct->Dim);
  Dim=IntVector(MyMultiStruct->Dim);
  Xdim=IntVector(MyMultiStruct->Dim);
  
  NewStruct=NewMultiStruct(MyMultiStruct->FileName,
			   MyMultiStruct->Dim,
			   NewSize,
			   MyMultiStruct->ArrayType);
  
  /* Evaluate starting points */
  ProductOfDim=1;
  for(j=0;j<MyMultiStruct->Dim;j++)
  {
    if(MyMultiStruct->Samples[j]>NewSize[j]) 
    { /* Shrink */
      Dim[j]=NewSize[j];
      LowerOut[j]=0;
      switch(NewOrder[j])
      {
	case -1:
	  LowerIn[j]=0;
	  break;
	case 0:
	  LowerIn[j]=floor((MyMultiStruct->Samples[j]-NewSize[j])/2);
	  if(!(MyMultiStruct->Samples[j]%2)&&((NewSize[j]%2))) 
	    LowerIn[j]++;
	  break;
	case 1:
	  LowerIn[j]=MyMultiStruct->Samples[j]-NewSize[j];
	  break;
	default:
	  Error("Unknown Order (ResizeMultiStruct)");
      }
    }
    else
    { /* Stretch */
      Dim[j]=MyMultiStruct->Samples[j];
      LowerIn[j]=0;
      switch(NewOrder[j])
      {
	case -1:
	  LowerOut[j]=0;
	  break;
	case 0:
	  LowerOut[j]=floor((NewSize[j]-MyMultiStruct->Samples[j])/2);
	  if((MyMultiStruct->Samples[j]%2)&&(!(NewSize[j]%2))) 
	    LowerOut[j]++;
	break;
	  case 1:
	  LowerOut[j]=NewSize[j]-MyMultiStruct->Samples[j];
	  break;
	default:
	  Error("Unknown Order (ResizeMultiStruct)");
      } 
    }
    ProductOfDim*=Dim[j];
  }

  /* Copying data */
  for(i=0;i<ProductOfDim;i++)
  {
    tmp=i;
    for(j=0;j<MyMultiStruct->Dim;j++)
    {
      Xdim[j]=tmp%Dim[j];
      tmp/=Dim[j];
    }
    
    Number_New=0;
    Number_Old=0;
    for(j=MyMultiStruct->Dim-1;j>=0;j--)
    {
      Number_New*=NewSize[j];
      Number_New+=(Xdim[j]+LowerOut[j]);
     
      Number_Old*=MyMultiStruct->Samples[j];
      Number_Old+=(Xdim[j]+LowerIn[j]);
    }


    if(NewStruct->ArrayType==_RealArray)
      NewStruct->Signal[Number_New]=MyMultiStruct->Signal[Number_Old];
    else
    {
      NewStruct->Signal[2*Number_New  ]=MyMultiStruct->Signal[2*Number_Old  ];
      NewStruct->Signal[2*Number_New+1]=MyMultiStruct->Signal[2*Number_Old+1];
    }
  }
    
  /* Update `->Min' */
  for(j=0;j<MyMultiStruct->Dim;j++)
  {
    NewStruct->Min[j]=MyMultiStruct->Min[j]+
      (LowerIn[j]-LowerOut[j])*MyMultiStruct->Delta[j];
    NewStruct->Delta[j]=MyMultiStruct->Delta[j];
  }
  /* Cleaning */
  Free(LowerIn);
  Free(LowerOut);
  Free(Dim);
  Free(Xdim);
  
  return (NewStruct);
}


/****************************************************************
[NAME]
FindSignalIndexInMultiStruct

[SYNOPSIS]
int FindSignalIndexInMultiStruct(int *Indices,int *Samples,int Dimensions)

[DESCRIPTION]

Returns index in {\tt MultiStruct->Signal} for the multidimensional element
with indices {\tt Indices}. Applies to {\tt MultiStruct} of type 
{\tt RealArray}. 

[USAGE]
{\tt Myindex=int FindSignalIndexInMultiStruct(Indices,Samples,Dim)}

[NOTE]
See {\tt FindIndicesInMultiStruct} for the opposite function.

[REVISION]
20/5-95 Peter Toft
****************************************************************/
int FindSignalIndexInMultiStruct(int *Indices,int *Samples,int Dimensions)
{
  int index,dim;
  
  index=Indices[Dimensions-1];
  for (dim=Dimensions-2;dim>-1;dim--)
    index=Indices[dim]+Samples[dim]*index;
  
  return index;
}


/****************************************************************
[NAME]
FindIndicesInMultiStruct

[SYNOPSIS]
void FindIndicesInMultiStruct(int *Indices,int *Samples,
			      int index,int Dimensions)

[DESCRIPTION]

Returns indeces in for the multidimensional element with index 
{\tt Index} in {\tt MultiStruct->Signal}. Applies to {\tt MultiStruct} of
type {\tt RealArray}.

[USAGE]
{\tt FindSignalIndexInMultiStruct(Indices,Samples,index,Dim)}

[NOTE]
See {\tt FindIndexInMultiStruct} for the opposite function.

[REVISION]
20/5-95 Peter Toft
****************************************************************/
void FindIndicesInMultiStruct(int *Indices,int *Samples,
			 int index,int Dimensions)
{
  int dim,newindex;
  
  for (dim=0;dim<Dimensions-1;dim++)
  {    
    newindex=(int)floor(index/Samples[dim]);
    Indices[dim]=index-Samples[dim]*newindex;
    index=newindex;
  }
  Indices[Dimensions-1]=index;
}


/****************************************************************
[NAME]
SliceMultiStruct

[SYNOPSIS]
MultiStruct * SliceMultiStruct(MultiStruct *InMultiStruct,int * CutVector)

[DESCRIPTION]

The function returns a slice of {\tt InMultiStruct} corresponding
to {\tt CutVector} which is of same dimension. Is the i'th element
of {\tt CutVector} is negative that dimenion is unchanged. If 
positive elements with that position is chosen. The number of 
non-negative elements determine the dimension of the returned
MultiStruct.

[USAGE]
{\tt MyMulti=SliceMultiStruct(InMultiStruct,CutVector)}

[REVISION]
20/5-95 Peter Toft
****************************************************************/
MultiStruct * SliceMultiStruct(MultiStruct *InMultiStruct,int * CutVector)
{
  int i,j,InDim,OutDim,inindex,outindex;
  int *OutSamples,*InIndices,*OutIndices;
  MultiStruct *OutMultiStruct;

  InDim=InMultiStruct->Dim;
  OutSamples=IntVector(InDim);
  InIndices=IntVector(InDim);
  OutIndices=IntVector(InDim);

  OutDim=0;
  for (i=0;i<InDim;i++)
  {
    if (CutVector[i]<0)
      OutSamples[OutDim++]=InMultiStruct->Samples[i];
    else
    {
      if (CutVector[i]>=InMultiStruct->Samples[i])
	Error("Bad CutVector index %i = %i (SliceMultiStruct)",
	      i,CutVector[i]);
    }
  }
  if (OutDim>1)
    Print(_DDebug,"Making a %i dimensional MultiStruct (SliceMultiStruct)\n",
	  OutDim);
  else
    Error("Must have one dimension free. Found zero (SliceMultiStruct)\n");
  OutMultiStruct=NewMultiStruct("SliceMultiStruct",
				OutDim,
				OutSamples,
				InMultiStruct->ArrayType);

  j=0;
  for (i=0;i<InDim;i++)
  {
    if (CutVector[i]>=0)
    {
      OutMultiStruct->Delta[j]=InMultiStruct->Delta[i];
      OutMultiStruct->Min[j++]=InMultiStruct->Min[i];
    }
  }    

  if (InMultiStruct->ArrayType==_RealArray)
    for (outindex=0;outindex<OutMultiStruct->NoOfSamples;outindex++)
    {
      FindIndicesInMultiStruct(OutIndices,OutSamples,outindex,OutDim);
      j=0;
      for (i=0;i<InDim;i++)
	if (CutVector[i]<0)
	  InIndices[i]=OutIndices[j++];
	else
	  InIndices[i]=CutVector[i];
      inindex=FindSignalIndexInMultiStruct(InIndices,
					   InMultiStruct->Samples,
					   InMultiStruct->Dim);
      OutMultiStruct->Signal[outindex]=InMultiStruct->Signal[inindex];
    }
  else
    for (outindex=0;outindex<OutMultiStruct->NoOfSamples;outindex++)
    {
      FindIndicesInMultiStruct(OutIndices,OutSamples,outindex,OutDim);
      j=0;
      for (i=0;i<InDim;i++)
	if (CutVector[i]<0)
	  InIndices[i]=OutIndices[j++];
	else
	  InIndices[i]=CutVector[i];
      inindex=FindSignalIndexInMultiStruct(InIndices,
					   InMultiStruct->Samples,
					   InMultiStruct->Dim);
      OutMultiStruct->Signal[2*outindex]=InMultiStruct->Signal[2*inindex];
      OutMultiStruct->Signal[2*outindex+1]=InMultiStruct->Signal[2*inindex+1];
    }
  
  Free(OutSamples);
  Free(InIndices);
  Free(OutIndices);
  
  return OutMultiStruct;
}


/****************************************************************
[NAME]
SubtractMultiStruct

[SYNOPSIS]
MultiStruct * SubtractMultiStruct(MultiStruct *,MultiStruct *);

[DESCRIPTION]
This function will subtract the second {\tt MultiStruct} from the first.
It is checked whether the two are of same size. 

[USAGE]
{\tt DifNewMultiStruct=SubtractMultiStruct(MyMult1,MyMult2);}

[REVISION]
30/6-95 Peter Toft\\
30/8-95 Peter Toft. More checks.
****************************************************************/
MultiStruct* SubtractMultiStruct(MultiStruct *MyMult1,
                                 MultiStruct *MyMult2)
{
  int m;
  MultiStruct *MyNewMultiStruct;
  float *sigout,*sig1,*sig2;

  Print(_DDebug,"SubtractMultiStruct\n");

  if (MyMult1->Dim!=MyMult1->Dim)
    Error("The two MultiStruct has different size (SubtractMultiStruct)");
  
  for (m=0;m<MyMult1->Dim;m++)
    if (MyMult1->Samples[m]!=MyMult1->Samples[m])
      Error("The two MultiStruct has different size (SubtractMultiStruct)");

  MyNewMultiStruct=NewMultiStruct(MyMult1->FileName,
				  MyMult1->Dim,
				  MyMult1->Samples,
				  MyMult1->ArrayType);

  for (m=0;m<MyNewMultiStruct->Dim;m++)
  {
    MyNewMultiStruct->Delta[m]=MyMult1->Delta[m];
    MyNewMultiStruct->Min[m]=MyMult1->Min[m];
  }

  sigout=MyNewMultiStruct->Signal;
  sig1=MyMult1->Signal;
  sig2=MyMult2->Signal;
  for (m=0;m<MyMult1->NoOfSamples*MyMult1->ArrayType;m++)
    sigout[m]=sig1[m]-sig2[m];

  return MyNewMultiStruct;
}


/****************************************************************
[NAME]
L2\_measure

[SYNOPSIS]
float * L2_measure(MultiStruct *,MultiStruct *);

[DESCRIPTION]
This function will return the variance estimate of the difference, 
i.e., the squareroot of the second order measure of misfit between 
the two {\tt MultiStruct}s. It is checked whether the two are of 
same size. 

[USAGE]
{\tt MyL2=L2\_measure(MyMult1,MyMult2);}

[REVISION]
25/7-95 Peter Toft
****************************************************************/
float L2_measure(MultiStruct *MyMult1,
                 MultiStruct *MyMult2)
{
  int m;
  float L2,dif;
  float *sig1,*sig2;

  Print(_DDebug,"L2_measure\n");

  if (MyMult1->Dim!=MyMult1->Dim)
    Error("The two MultiStruct has different size (L2_measure)");

  if (MyMult1->ArrayType==_ComplexArray)
    Error("This function only applies to real valued signals (L2_measure)");

  sig1=MyMult1->Signal;
  sig2=MyMult2->Signal;
  L2=0.0;
  for (m=0;m<MyMult1->NoOfSamples;m++)
  {
    dif=sig1[m]-sig2[m];
    L2+=dif*dif;
  }
  L2=sqrt(L2/(MyMult1->NoOfSamples-1.0));

  return L2;
}


/****************************************************************
[NAME]
StatMultiStruct

[SYNOPSIS]
void StatMultiStruct(MultiStruct *,float *, float*);

[DESCRIPTION]
This function will return the mean estimate and the deviance
estimate of the {\tt MultiStruct}.

[USAGE]
{\tt StatMultiStruct(MyMulti,\&MyMean,\&MyDev);}

where {\tt MyMean, MyDev} are of type {\tt float}.

[REVISION]
25/7-95 Peter Toft
****************************************************************/
void StatMultiStruct(MultiStruct *MyMulti,float *MyMean,float *MyDev)
{
  int m;
  float Sum,Sum2,sigvalue;
  float *sig1;

  Print(_DDebug,"StatMultiStruct\n");

  if (MyMulti->ArrayType==_ComplexArray)
    Error("This function applies to real valued signals (StatMultiStruct)");

  sig1=MyMulti->Signal;
  Sum=Sum2=0.0;

  for (m=0;m<MyMulti->NoOfSamples;m++)
  {
    sigvalue=sig1[m];
    Sum+=sigvalue;
    Sum2+=sigvalue*sigvalue;
  }

  *MyMean=Sum/MyMulti->NoOfSamples;
  *MyDev=sqrt((Sum2-Sum*Sum/(MyMulti->NoOfSamples))/(MyMulti->NoOfSamples));
}

