#include <math.h>
#include <time.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <malloc.h>
#include "3dtools.h"
#include "3dmisc.h"
#include "FIF_IO.h"
#include "Multi_IO.h"


/***************************************************************************
[NAME]
NewFloatImage

[SYNOPSIS]
Image *NewFloatImage(char *FileName,
                     int M,
                     int N,
                     int ArrayType);

[DESCRIPTION] 

This function allocates and returns a float matrix of size {\tt M
$\times$ N}.  Initializes the variables {\tt N}, {\tt M}, {\tt
FileName} and {\tt Arraytype} in the Image structure.  {\tt ArrayType}
can be {\tt \_ComplexArray} or {\tt \_RealArray}.  The size of the
array is number of elements, either complex or real. The new image is
initialized to the value 0.

[USAGE] 

{\tt NewImage=NewFloatImage("test",7,8,\_RealArray);} 

Allocates a 7 $\times$ 8 matrix with real numbers.

[REVISION]
Oct. 94, JJJ and PAP
***************************************************************************/
Image * NewFloatImage(char * FileName,int M,int N,int ArrayType)
{
  int m;
  Image * NewImage;

  Print(_DDebug,"NewFloatImage: Allocating memory for `%s'\n", FileName);

  if(!(NewImage=(Image *) malloc(sizeof(Image)))) 
    Error( "Memory allocation problems (NewFloatImage)" );

  if(!(NewImage->Signal=(float **) malloc(M*sizeof(float *)))) 
    Error("Memory allocation problems (NewFloatImage)" );

  for (m=0;m<M;m++)
    NewImage->Signal[m]=FloatVector((ArrayType*N));
  
  strcpy(NewImage->FileName,FileName);
  NewImage->M=M;
  NewImage->N=N;
  NewImage->Xmin=0;
  NewImage->Ymin=0;
  NewImage->DeltaX=1;
  NewImage->DeltaY=1;

  NewImage->ArrayType=ArrayType;

  NewImage->FIFIdType=_00EI;
  
  return NewImage;
}


/***************************************************************************
[NAME]
FreeImage

[SYNOPSIS]
void FreeImage(Image *MyImage);

[DESCRIPTION]
This function frees the memory allocated for {\tt MyImage}.
							 
[USAGE]
{\tt FreeImage(Test);}

Frees the memory allocated for {\tt test}.

[REVISION]
Oct. 94, JJJ and PAP
***************************************************************************/
void FreeImage(Image * MyImage)
{
  int m;

  Print(_DDebug,"FreeImage: Freeing `%s'\n", MyImage->FileName);

  for (m=0;m<MyImage->M;m++) 
    Free(MyImage->Signal[m]);
  Free(MyImage->Signal);
  Free(MyImage);
}


/***************************************************************************
[NAME]
ReadFIF

[SYNOPSIS]
Image *ReadFIF(char *FileName);

[DESCRIPTION]

The function reads a picture in {\tt .fif} format (raw float with
header) from the file {\tt 'FileName.fif'}. All memory allocation is
done internally. A pointer to the new image is returned.

[USAGE]
{\tt Test = ReadFIF("Test");}

Reads the image {\tt Test.fif} and return the pointer in {\tt Test}.

[REVISION]
Oct. 94, JJJ and PAP
***************************************************************************/
Image * ReadFIF(char *FileName)
{
  int m,n,DoConvert;
  FILE *InFile;
  char InFileName[100];
  Image *NewImage,*TempImage;

  DoConvert=FALSE;
    
  strcpy(InFileName,FileName);
  strcat(InFileName,".fif");

  Print(_DNormal,"ReadFIF: Reading `%s' \n",InFileName);
  
  if(!(InFile=fopen(InFileName,"rb")))	
    Error("Error opening file: `%s'",InFileName);
  
  if(!(NewImage=(Image *) malloc(sizeof(Image)))) 
    Error( "\nMemory allocation problems (ReadFIF)\n" );

  fread(NewImage,sizeof(Image),1,InFile);

  if(NewImage->FIFIdType!=_00EI) {
    DoConvert=TRUE;
    convert4((char *)&NewImage->FIFIdType);
    if(NewImage->FIFIdType!=_00EI) Error("Error reading FIF (Converting)\n"); 
  }
  
  /* Converting FIF header.*/
  if(DoConvert) {
    convert4((char *)&NewImage->Type);
    convert4((char *)&NewImage->M);
    convert4((char *)&NewImage->N);
    convert4((char *)&NewImage->ArrayType);
    convert4((char *)&NewImage->Xmin);
    convert4((char *)&NewImage->Ymin);
    convert4((char *)&NewImage->DeltaX);
    convert4((char *)&NewImage->DeltaY);
    convert4((char *)&NewImage->SignalMin);
    convert4((char *)&NewImage->SignalMax);
  }
  /*                       */

  TempImage=NewFloatImage(FileName,NewImage->M,NewImage->N,NewImage->ArrayType);
  NewImage->Signal=TempImage->Signal;
  Free(TempImage);

  for(m=0;m<NewImage->M;m++)
    if((fread(NewImage->Signal[m],sizeof(float),
        NewImage->N*NewImage->ArrayType,InFile))!=NewImage->N*NewImage->ArrayType)
      Error("Error reading file: `%s'",InFileName);
  
  if(DoConvert) 
    for(m=0;m<NewImage->M;m++)
      for(n=0;n<(NewImage->N*NewImage->ArrayType);n++)
	convert4((char *)&NewImage->Signal[m][n]);
  
  
  fclose(InFile);
  return NewImage;
}


/***************************************************************************
[NAME]
WriteFIF

[SYNOPSIS]
void WriteFIF(Image *MyImage);

[DESCRIPTION]

The function writes a picture in {\tt .fif} format. The name of the
saved image is determined by {\tt MyImage->FileName}. The file
extension is appended to the filenames in all the `write' routines.

[USAGE]
{\tt WriteFIF(Test);}

Write the image {\tt Test} to a {\tt .fif} file .

[REVISION]
Oct. 94, JJJ and PAP
***************************************************************************/
void WriteFIF(Image * MyImage)
{
  int m;
  FILE *OutFile;
  char OutFileName[100];

  strcpy(OutFileName,MyImage->FileName);
  strcat(OutFileName,".fif");

  Print(_DNormal,"WriteFIF: Writing `%s' (%dx%d) \n",
          OutFileName,MyImage->M,MyImage->N);

  if(!(OutFile=fopen(OutFileName,"wb"))) 
    Error("Error opening file: `%s'",OutFileName);

  fwrite(MyImage,sizeof(Image),1,OutFile);

  for(m=0;m<MyImage->M;m++) 
    if((fwrite(MyImage->Signal[m],sizeof(float),
        MyImage->N*MyImage->ArrayType,OutFile))!=MyImage->N*MyImage->ArrayType)
      Error("Error writing to file: `%s'",OutFileName);

  fclose(OutFile);
}

 
/****************************************************************
[NAME]
F2F\_to\_Image

[SYNOPSIS]
Image * F2F_to_Image(MultiStruct *My2D)

[DESCRIPTION]

This function will convert a two-dimensional {\tt MultiStruct}, 
i.e., a {\tt .f2f}-file to the {\tt fif}-format. Note that the
{\tt sigmin} and {\tt sigmax} fields are not updated

[USAGE]
{\tt MyImage=F2F\_to\_Image(My2DMultiStruct);}

[REVISION]
20/5-95 Peter Toft
****************************************************************/
Image * F2F_to_Image(MultiStruct *My2D)
{
  int xint,yint,XSamples;
  Image * MyImage;

  if (My2D->Dim!=2)
    Error("Input MultiStruct is not 2D (F2F_to_Image)\n");

  XSamples=My2D->Samples[0];
  MyImage=NewFloatImage(My2D->FileName,
			My2D->Samples[0],
			My2D->Samples[1],
			My2D->ArrayType);
  MyImage->Xmin=My2D->Min[0];
  MyImage->Ymin=My2D->Min[1];
  MyImage->DeltaX=My2D->Delta[0];
  MyImage->DeltaY=My2D->Delta[1];
  if (My2D->ArrayType==_RealArray)
  {
    for (xint=0;xint<My2D->Samples[0];xint++)
      for (yint=0;yint<My2D->Samples[1];yint++)
	MyImage->Signal[xint][yint]=My2D->Signal[xint+XSamples*yint];
  }
  else
  {
    for (xint=0;xint<My2D->Samples[0];xint++)
      for (yint=0;yint<My2D->Samples[1];yint++)
      {
	MyImage->Signal[xint][2*yint]=My2D->Signal[2*(xint+XSamples*yint)];
	MyImage->Signal[xint][2*yint+1]=My2D->Signal[2*(xint+XSamples*yint)+1];
      }
  }
  return MyImage;
}


/****************************************************************
[NAME]
Image\_to\_F2F

[SYNOPSIS]
MultiStruct * Image_to_F2F(Image * MyImage)

[DESCRIPTION]

This function will convert an image in {\tt fif}-format to a 
two-dimensional {\tt MultiStruct}, 
i.e., a {\tt .fif}-file.

[USAGE]
{\tt My2DMultiStruct=Image\_to\_F2F(MyImage);}

[REVISION]
20/5-95 Peter Toft
****************************************************************/
MultiStruct * Image_to_F2F(Image * MyImage)
{
  int xint,yint;
  MultiStruct * My2D;
  int Samples[2];
  int XSamples;

  XSamples=Samples[0]=MyImage->M;
  Samples[1]=MyImage->N;
  My2D=NewMultiStruct(MyImage->FileName,2,Samples,MyImage->ArrayType);

  My2D->Min[0]=MyImage->Xmin;
  My2D->Min[1]=MyImage->Ymin;
  My2D->Delta[0]=MyImage->DeltaX;
  My2D->Delta[1]=MyImage->DeltaY;
  if (MyImage->ArrayType==_RealArray)
  {
    for (xint=0;xint<My2D->Samples[0];xint++)
      for (yint=0;yint<My2D->Samples[1];yint++)
	My2D->Signal[xint+XSamples*yint]=MyImage->Signal[xint][yint];
  }
  else
  {
    for (xint=0;xint<My2D->Samples[0];xint++)
      for (yint=0;yint<My2D->Samples[1];yint++)
      {
	My2D->Signal[2*(xint+XSamples*yint)]=MyImage->Signal[xint][2*yint];
	My2D->Signal[2*(xint+XSamples*yint)+1]=MyImage->Signal[xint][2*yint+1];
      }
  }
  return My2D;
}


/****************************************************************
[NAME]
FIF\_to\_F3F

[SYNOPSIS]
MultiStruct* FIF_to_F3F(char *InFileName,
			int StartSlice,
			int NumberOfSlices,
			float DeltaZ,
			float ZMin)

[DESCRIPTION]

The function will read the files {\tt Infilename\_<StartSlice>.fif}
until {\tt Infilename\_<StartSlice+NumberOfSlices-1>.fif} and stack
(not sum) the images in the z-direction (third coordinate). The sampling
distances are taken from the fif-files and the Sampling distance in
the z-direction is set to {\tt DeltaZ}.
 
[USAGE] 
{\tt MyMultiStruct=FIF\_to\_F3F("MyFIF",2,5,1.2,-1);}

This will read the files {\tt MyFIF\_2.fif} to {\tt MyFIF\_6.fif} and
return a stacked volume.


[NOTE]
Use {\tt Write\_FIFs\_to\_F3F} to get the MultiStruct.


[REVISION]
20/5-95 Peter Toft
****************************************************************/
MultiStruct* FIFs_to_F3F(char *InFileName,
			 int StartSlice,
			 int NumberOfSlices,
			 float DeltaZ,
			 float ZMin)
{

  int slice,i,j,ArrayType,XSamples,YSamples,off;
  Image *MyImage;
  char Filename[200];
  MultiStruct *MyMultiStruct;
  int *Samples;
  float *Signal3D;
  float *Signal2D;

  if (NumberOfSlices<=0) Error("Illegal number of slices (FIF_to_F3F)\n");
  Samples=IntVector(3);
  sprintf(Filename,"%s_%i",InFileName,StartSlice);
  MyImage=ReadFIF(Filename);
  ArrayType=MyImage->ArrayType;
  XSamples=Samples[0]=MyImage->M;
  YSamples=Samples[1]=MyImage->N;
  Samples[2]=NumberOfSlices;
  
  MyMultiStruct=NewMultiStruct(InFileName,3,Samples,ArrayType);
  Signal3D=MyMultiStruct->Signal;
  MyMultiStruct->Delta[0]=MyImage->DeltaX;
  MyMultiStruct->Delta[1]=MyImage->DeltaY;
  MyMultiStruct->Delta[2]=DeltaZ;
  MyMultiStruct->Min[0]=MyImage->Xmin;
  MyMultiStruct->Min[1]=MyImage->Ymin;
  MyMultiStruct->Min[2]=ZMin;

  for (slice=StartSlice;slice<StartSlice+NumberOfSlices;slice++)
  {
    if (slice!=StartSlice)
    {
      sprintf(Filename,"%s_%i",InFileName,slice);
      MyImage=ReadFIF(Filename);
    }
    if (ArrayType==_RealArray)
      for (i=0;i<XSamples;i++)
      {
	Signal2D=MyImage->Signal[i];
	for (j=0;j<YSamples;j++)
	  Signal3D[i+XSamples*(j+YSamples*(slice-StartSlice))]=Signal2D[j];
      }
    else
      for (i=0;i<XSamples;i++)
      {
	Signal2D=MyImage->Signal[i];
	for (j=0;j<YSamples;j++)
	{
	  off=2*(i+XSamples*(j+YSamples*(slice-StartSlice)));
	  Signal3D[off]=Signal2D[2*j];
	  Signal3D[off+1]=Signal2D[2*j+1];
	}
      }
    FreeImage(MyImage);
  }
  return MyMultiStruct;
}


/****************************************************************
[NAME]
Write\_FIFs\_to\_F3F

[SYNOPSIS]
void Write_FIFs_to_F3F(char *InFileName,
		       int StartSlice,
		       int NumberOfSlices,
		       float DeltaZ,
		       float ZMin)


[DESCRIPTION]

The function will read the files {\tt Infilename\_<StartSlice>.fif}
until {\tt Infilename\_<StartSlice+NumberOfSlices-1>.fif} and stack
(not sum) the images in the z-direction (third coordinate). The
sampling distances are taken from the fif-files and the Sampling
distance in the z-direction is set to {\tt DeltaZ}. The function will
write the volume, a {\tt .f3f}-file, to the disk. 

 
[USAGE] 
{\tt MyMultiStruct=FIF\_to\_F3F("MyFIF",2,5,1.2,-1);}

This will read the files {\tt MyFIF\_2.fif} to {\tt MyFIF\_6.fif} and
return a stacked volume.

[NOTE]
Use {\tt FIF\_to\_F3F} to get the MultiStruct.

[REVISION]
20/5-95 Peter Toft\\
2/6-95 Peter Toft (Changed OutFileName)
****************************************************************/
void Write_FIFs_to_F3F(char *InFileName,
		       int StartSlice,
		       int NumberOfSlices,
		       float DeltaZ,
		       float ZMin)
{
  int size,slice,i,j,ArrayType,XSamples,YSamples,off;
  Image *MyImage;
  char OutFileName[200],Filename[200];
  MultiStruct *MyMultiStruct;
  int *Samples;
  float *Signal3D;
  float *Signal2D;
  FILE *outfile;

  sprintf(OutFileName,"%s.f3f",InFileName);
  if (NumberOfSlices<=0) Error("Illegal number of slices (FIF_to_F3F)\n");
  Samples=IntVector(3);
  sprintf(Filename,"%s_%02i",InFileName,StartSlice);
  MyImage=ReadFIF(Filename);
  ArrayType=MyImage->ArrayType;
  XSamples=Samples[0]=MyImage->M;
  YSamples=Samples[1]=MyImage->N;
  Samples[2]=NumberOfSlices;
  size=XSamples*YSamples*ArrayType;
  Signal3D=FloatVector(size);
  MyMultiStruct=NewMultiStruct_NoSignal(InFileName,3,Samples,ArrayType);
  MyMultiStruct->Delta[0]=MyImage->DeltaX;
  MyMultiStruct->Delta[1]=MyImage->DeltaY;
  MyMultiStruct->Delta[2]=DeltaZ;
  MyMultiStruct->Min[0]=MyImage->Xmin;
  MyMultiStruct->Min[1]=MyImage->Ymin;
  MyMultiStruct->Min[2]=ZMin;
  WriteMultiStruct_NoSignal(MyMultiStruct);
  outfile=fopen(OutFileName,"ab");

  for (slice=StartSlice;slice<StartSlice+NumberOfSlices;slice++)
  {
    if (slice!=StartSlice)
    {
      sprintf(Filename,"%s_%02i",InFileName,slice);
      MyImage=ReadFIF(Filename);
    }
    if (ArrayType==_RealArray)
      for (i=0;i<XSamples;i++)
      {
	Signal2D=MyImage->Signal[i];
	for (j=0;j<YSamples;j++)
	  Signal3D[i+XSamples*j]=Signal2D[j];
      }
    else
      for (i=0;i<XSamples;i++)
      {
	Signal2D=MyImage->Signal[i];
	for (j=0;j<YSamples;j++)
	{
	  off=2*(i+XSamples*j);
	  Signal3D[off]=Signal2D[2*j];
	  Signal3D[off+1]=Signal2D[2*j+1];
	}
      }
    FreeImage(MyImage);
    fwrite(Signal3D,sizeof(float),size,outfile);
  }
  fclose(outfile);
}
